;-----------------------------------------------------------------------------
;NAME
;   auto-arg.ml -- invert the function of ^U
;
;SYNOPSIS
;   No function entry points
;
;KEY BINDINGS
;   See below.
;
;DESCRIPTION
;	This package implements automatic argument  generation.  Whenever
;	a number is typed, it is interpreted as if ^U had been hit first,
;	except that  the argument  appears in  the minibuffer.   When the
;	first non-digit character is hit,  the action  is taken.   If the
;	next  character is  a command,  it is  done the  number of times.
;	Otherwise the number is  inserted into  the text  followed by the
;	character that terminated the argument.  
;
;	The  only  functions  important  to  users   are  "auto-arg"  and
;	"no-auto-arg".  "auto-arg"  sets all  the local  key bindings and
;	turns  the package  on.   "no-auto-arg" removes  all the bindings
;	that "auto-arg" set up and turns the package  off.  Theoretically
;	this will leave any local mode intact.
;
;       "neg-arg" - is bound to '-' and allows negative prefix args.
; 
;	"auto-arg-insert"  is  bound  to  all  self-inserting  characters
;	except for '-'.   This  function just  checks the  prefix arg and
;	then inserts the character.
;
;	"n-digit" is bound to the numbers.  It does the bulk  of the work
;	that is described below.
;
; HISTORY
;   26-Aug-84 mcl (John S Mc Laughlin at Unipress Software)
;	     Added no-auto-arg to turn off the package.
;	     Made everything local-bindings.
;   ??-Aug-84 mcl (John S Mc Laughlin at Unipress Software)
;	      Made the progn at the end of this file (defun (auto-arg ... ) )
;	      added defuns for each procedure insted of one big defun
;   05-Jul-84  Eric S. Raymond at Rabbit Software
;	Added this documentation.
; ---------------------------------------------------------------------------

(if (! (is-bound is-neg))
   (setq-default is-neg 0)
)

(defun
  (neg-arg
    (setq is-neg 1)
    (message "Arg: -")
    (return-prefix-argument 0)
  )

  (auto-arg-insert
    (if prefix-argument-provided
       (insert-string prefix-argument)
    )
    (self-insert)
  )

  (n-digit n
    (if (! prefix-argument-provided)
       (setq is-neg 0)
    )
    (if is-neg
       (setq n (- (* (?: prefix-argument-provided prefix-argument 0) 10)
			  (% (last-key-struck) 16)))
       (setq n (+ (* (?: prefix-argument-provided prefix-argument 0) 10)
			  (% (last-key-struck) 16)))
    )
    (message "Arg: " n)
    (return-prefix-argument n)
  )

  (auto-arg i
    (setq i ' ')
    (while (< i 0177)
       (local-bind-to-key "auto-arg-insert" i)
       (++ i)
    )
    (setq i '0')
    (while (<= i '9')
       (local-bind-to-key "n-digit" i)
       (++ i)
    )
    (local-bind-to-key "neg-arg" "-")
  )

  (no-auto-arg
    (progn i
      (setq i 127)
      (while (- i 31)
         (remove-local-binding (-- i))
      )
    )
  )
)
