;-----------------------------------------------------------------------------
;NAME
;   crypt.ml -- read and write crypted files using crypt(1).
;
;SYNOPSIS
;   crypt		-- begin/end crypt mode
;   read-crypted-file	-- decrypt a file into current buffer
;   visit-crypted-file	-- visit a crypted file
;   insert-crypted-file	-- decrypt a file, insert at dot
;   write-crypted-file	-- write encrypted buffer to named file
;   save-crypted-file	-- encrypts current buffer to associated filename
;
;KEY BINDINGS
;   read-crypted-file		read-file		^X^R
;   visit-crypted-file		visit-file		^X^V
;   insert-crypted-file		insert-file		^X^I
;   write-crypted-file		write-named-file	^X^W
;   save-crypted-file		write-current-file	^X^S
;   
;DESCRIPTION
;   The crypt function rebinds the normal emacs key sequences for file reading
;and writing to a set that handles encrypted files (see above). All of these
;except save-crypted-file prompt for a filename and password (which is actually
;the encryption key). Calling crypt while in crypt mode reverses these rebinds
;and restores the normal functions.
;
;   By default, crypt mode bypasses the normal backup mechanisms so as not
;to leave extra versions of encrypted things lying around. Every time a
;crypted file is written the old version (if any) is removed beforehand (which
;could be trouble in the unlikely event of a crash between the two actions).
;   This action can be prevented by setting delete-old-crypted-files to zero.
;
;   Also, 'group' and 'other' permissions are removed with chmod(1) every time
;an encrypted file is written.
;
;   All these functions leave mark where they found it. The only system
;variable crypt mode messes with is 'needs-checkpointing', which it restores
;on exit.
;
;SEE ALSO
;   crypt(1), makekey(8).
;
;HISTORY
; 04-Jul-84  Eric S. Raymond (eric) at Rabbit Software.
;	Loosely based on Agin at CMU's original.
; 11-Aug-84  John S. Mc Laughlin (McL) at UniPress Software Inc.
;	Changed filter-region to fast-filter-region for speed.
;
;-----------------------------------------------------------------------------

(setq-default crypt-toggle 0)

(declare-buffer-specific crypt-file-name)
(declare-buffer-specific crypt-password)
(declare-buffer-specific crypt-toggle)
(declare-buffer-specific old-checkpoint-value)
(declare-buffer-specific old-mode)

(if (! (is-bound delete-old-crypted-files))
    (setq-default delete-old-crypted-files 1))
(if (! (is-bound checkpoint-crypted-files))
    (setq-default checkpoint-crypted-files 0))


(defun
    (crypt
	(if crypt-toggle
	    (progn
		(setq needs-checkpointing old-checkpoint-value)
		(setq mode-string old-mode)
		(local-rebind "nothing" "read-crypted-file" "\^X\^R")
		(local-rebind "nothing" "visit-crypted-file" "\^X\^V")
		(local-rebind "nothing" "insert-crypted-file" "\^X\^I")
		(local-rebind "nothing" "save-crypted-file" "\^X\^S")
		(local-rebind "nothing" "write-crypted-file" "\^X\^W")
	    )
	    (progn
		(setq old-checkpoint-value needs-checkpointing)
		(setq needs-checkpointing checkpoint-crypted-files)
		(setq old-mode mode-string)
		(setq mode-string "crypt")
		(local-rebind "read-crypted-file" "read-file" "\^X\^R")
		(local-rebind "visit-crypted-file" "visit-file" "\^X\^V")
		(local-rebind "insert-crypted-file" "insert-file" "\^X\^I")
		(local-rebind "save-crypted-file" "write-current-file" "\^X\^S")
		(local-rebind "write-crypted-file" "write-named-file" "\^X\^W")
	    )
	)
    )

    (visit-crypted-file
	vname
	(setq vname (expand-file-name (arg 1 "Visit crypted file: ")))
	(pop-to-buffer (root-name vname))
	(if (!= (buffer-size) 0)
	    (error-message "File is already in buffer")
	)
	(setq crypt-file-name vname)
	(if (! (file-exists vname))
	    (error-message (concat "New file: " vname))
	)
	(insert-crypted-file vname)
	(beginning-of-file)
	(setq buffer-is-modified 0)
	(novalue)
    )

    (read-crypted-file
	rname
	(setq rname (expand-file-name (arg 1 "Read crypted file: ")))
	(if (! (file-exists rname))
	    (error-message (concat "Can't find " rname))
	)
	(if (!= (buffer-size) 0)
	    (if (!= (substr (get-tty-string 
			"Do you really want to clobber text in this buffer? ")
			    1 1)
		    "y"
		)
		(error-message "Aborted.")
		(progn
		      (beginning-of-file)
		      (push-mark)
		      (end-of-file)
		      (delete-to-killbuffer)
		      (pop-mark)
		)
	    )
	)
	(setq crypt-file-name rname)
	(insert-crypted-file rname)
	(beginning-of-file)
	(setq buffer-is-modified 0)
	(novalue)
    )

    (insert-crypted-file
	iname
	(setq iname (expand-file-name (arg 1 "Insert crypted file: ")))
	(if (! (file-exists iname))
	    (error-message (concat "Can't find " iname))
	)
	(read-password-once)
	(push-mark)
	(fast-filter-region (concat "crypt " crypt-password " < " iname))
	(pop-mark)
	(novalue)
    )

    (write-crypted-file
	wname
	(setq wname (expand-file-name (arg 1 "Write crypted file: ")))
	(read-password-twice)
	(setq crypt-file-name wname)
	(save-crypted-file)
	(novalue)
    )

    (save-crypted-file
	spass sfile
	(if (= crypt-file-name "")
	    (error-message "No crypt file name to save under")
	)
	(if (= crypt-password "")
	    (read-password-twice)
	)
	(if (& delete-old-crypted-files (file-exists crypt-file-name))
	    (progn
		(unlink crypt-file-name)
		(message "Deleted " crypt-file-name)
		(sit-for 0)
	    )
	)
	(setq sfile crypt-file-name)
	(setq spass crypt-password)
	(save-excursion
	    (beginning-of-file)
	    (push-mark)
	    (end-of-file)
	    (copy-region-to-buffer "crypt-temp")
	    (temp-use-buffer "crypt-temp")
	    (beginning-of-file)
	    (set-mark)
	    (end-of-file)
	    (fast-filter-region (concat "crypt " spass " > " sfile))
	    (fast-filter-region (concat "chmod o-r,g-r " sfile))
	    (pop-mark)
	)
	(delete-buffer "crypt-temp")
	(setq buffer-is-modified 0)
	(message crypt-file-name " is encrypted.")
	(novalue)
    )

    (read-password-once pass
	(setq pass (get-password "Password:"))
	(if (= pass "") (error-message "Aborted."))
	(setq crypt-password pass)
    )

    (read-password-twice pass
	(setq pass (get-password "Password:"))
	(if (= pass "") (error-message "Aborted."))
	(if (!= pass (get-password "Retype password:"))
	    (error-message "Passwords disagree -- Aborted."))
	(setq crypt-password pass)
    )

    (get-password
	c password more mess
	(setq mess (arg 1))
	(setq password "")
	(setq more 1)
	(while more
	       (message mess)
	       (setq c (get-tty-character))
	       (if (= c '\r') (setq more 0)
		   (= c '^G') (progn
				    (setq password "")
				    (setq more 0)
			      )
		   (| (= c '\177') (= c '^H'))
			      (setq password (substr password 1 -1))
		   (setq password (concat password (char-to-string c)))
	       )
	)
	password
    )

    (root-name name
	i
	(setq name (arg 1))
	(setq i 1)
	(while (& (<= i (length name))
		  (!= (substr name (- 0 i) 1) "/")
	       )
	       (setq i (+ 1 i))
	)
	(setq i (- i 1))
	(substr name (- 0 i) i)
    )
)

; crypt.ml ends here ---------------------------------------------------------

