;----------------------------------------------------------------------------
; debug.ml -- MLisp debugging routines
;
;SYNOPSIS
;   debug		-- toggle compiler's debugging-hook generation
;   show-error		-- visit MLisp source line where last error occurred
;   single-step-handler	-- interactive debugger for MLisp execution
;
;KEY BINDINGS
;   show-error		next-error		ESC-S
;
;DESCRIPTION
; this file is the mlisp part of the debugging package.
;
; debug - toggles the variable compile-mlisp-debugging-hooks.
; 
; show-error - visits the file where the last mlisp error occured and places
; 	       the cursor at the point of the error.
; 
; single-step-handle - this is not a user-callable function.  This is called
; 		       by the single-step primitives (the single step
;		       primitives are internal magic) and does the cursor
; 		       location and parses the single-step commands
; 
; breakpoint - this is typically called by (set "single-step" (breakpoint))
; 	       breakpoint will stop the execution of an mlisp function and
;	       allow you to check the value of variables and test code.
; 	       breakpoint will promt with Bpoint> and will the read a string
; 	       from the terminal and then call execute-mlisp-line on that
; 	       string.
; 
;     To use the single step facility call debug.  If 
; compile-mlisp-debugging-hooks was turned off you will get the
; message Packages that you load from now on will be debuggable,
; otherwise you will get the message 
; Packages that you load from now on won't be debugable.
; Packages that you wish to debug must be loaded any time after the
; call to debug.  Once the package has been loaded then to set single
; step mode call the primitive step-through and give it the name of
; the function to be stepped.  After all this when you call the
; function it will automatically be stepped through.  If you wish to
; start stepping only after a certain point in your program you can
; set the function specifc variable single-step, i.e. if you were to
; put (set "single-step" 1) and the function was load debuggable
; then at that point you will start single stepping.
;HISTORY
; 08-Aug-84    John McLaughlin  (mcl at unipress)
;		Made the mode line give the command info
; 26-Jul-84    John McLaughlin  (mcl at unipress) 
;		Added breakpoint
;
;   04-Jul-84  Eric S. Raymond at Rabbit Software
;	Changed package to use rebinds, added this documentation.
;----------------------------------------------------------------------------

(defun (debug
	     (setq compile-mlisp-debugging-hooks
		   (! compile-mlisp-debugging-hooks))
	     (message "Packages that you load from now on "
		 (if compile-mlisp-debugging-hooks "will" "won't")
		 " be debuggable")
	     (novalue)))

(defun (show-error debug_fn p
	   (setq debug_fn (break-file-name))
	   (setq p (break-character-position))
	   (if (= debug_fn "")
	       (message "The last error wasn't from an MLisp function.")
	       (progn (visit-file debug_fn)
		      (widen-region)
		      (goto-character p)))
	   (novalue)))

(defun (single-step-handler fn p c ldot ret continue
	   (setq ldot (dot))
	   (save-excursion omode
	       (setq omode mode-line-format)
	       (setq fn (break-file-name))
	       (setq p (break-character-position))
	       (visit-file fn)
	       (setq mode-line-format  "Stepper.  space->continue, s->skip, l->look, r->recurse, p->print")
	       (goto-character p)
	       (setq continue 1)
	       (while continue
		      (setq c (get-tty-character))
		      (if (= c ' ') (progn (setq ret 1)
					   (setq continue 0))
			  (= c 's') (progn (setq ret 0)
					   (setq continue 0))
			  (= c 'l') (save-excursion
					(pop-to-buffer ldot)
					(goto-character ldot)
					(message "Dot is here.")
					(sit-for 10))
			  (= c 'r') (save-excursion (recursive-edit))
			  (= c 'p') (error-occurred v
					(setq v (get-tty-string ": print "))
					(message v " = "
					    (execute-mlisp-line v))
					(sit-for 10))
;			  (= c 'g') (progn (set "single-step" 0))
			  (= c '\^G') (progn 
					(setq mode-line-format omode)
					(error-message "Giving up")
				      )
;			  (= c 'e') (save-excursion sexpr
;				      (error-occurred
;					(set-mark)
;					(forward-balanced-paren-line)
;					(region-to-string sexpr)
;					(temp-use-buffer "debug-scratch")
;					(message (execute-mlisp-line sexpr))
;					(delete-buffer "debug-scratch")
;					(get-tty-character)
;				      )
;				    )
			  (progn (message "Huh?")
				 (sit-for 10))
		      )
	       )
	       (setq mode-line-format omode)
	   )
	   (if ret (&step-through-arg-1)
	       (arg 1))
       ))

(defun
  (breakpoint enter ss omode
    (save-excursion
      (visit-file (break-file-name))
      (setq omode mode-line-format)
      (setq mode-line-format "Type exit to leave breakpoint or step for single stepping")
      (sit-for 90)
      (setq ss 0)
      (while (!= (substr (setq enter (get-tty-string "Bpoint> ")) 1 1) "e")
	(if (= enter "step")
	  (setq ss 1)
          (progn rv
            (if (error-occurred 
	      (setq rv (concat enter " -> " (execute-mlisp-line enter)))
		)
	      (progn 
	        (message (concat enter " caused an error!"))
	        (sit-for 30)
	      )
	      (progn 
	        (message rv)
	        (sit-for 30)
	      )
	    )
	  )
	)
      )
      (setq mode-line-format omode)
    )
    ss
  )
)
     

(global-rebind "show-error" "next-error" "\es")
(define-string-macro "bpoint" "(set ""single-step"" (breakpoint))")
; because of a bug in mlisp the above macro doesn't work.
(novalue)
