;----------------------------------------------------------------------------
; dired.ml -- a directory-editing package
;
;SYNOPSIS
;   dired	-- edit a directory through its listing
;
;KEY BINDINGS
;   n		-- go to next file (all next-line key sequences work)
;   p		-- go to previous file (all previous-line key sequences work)
;   d		-- mark file on this line for deletion on exit from dired
;   u		-- remove deletion mark from file
;   RUBOUT	-- go back to previous file, remove deletion mark
;   e, v	-- Recursive-edit the current file
;   r		-- remove current listing line (file won't be delete-marked)
;   q		-- exit dired, querying before performing deletions
;
;DESCRIPTION
; 
;       This package allows you to edit your  directory.  That is you
;  can veiw a file in a recursive edit or delete a  file or  undelete
;  it.  This function sets up all the files in a  buffer  and  does a
;  recursive edit.  The work functions are bound to keys in the local
;  keymap &dired-keymap.  
; 
;
;HISTORY
; Wed Aug 08 15:26:02 1984  John McLaughlin  (mcl at unipress)
;     fixed a bug in &dired-examine had to move to dot + 46 not dot + 43
;        also fixed the help line   July 25 1984
;
;   04-Jul-84  Eric S. Raymond at Rabbit Software
;	Added this documentation.
;----------------------------------------------------------------------------
;

(declare-global Dired-keymap-defined
                Dired-dir)

(setq Dired-keymap-defined 0)
(setq Dired-dir (working-directory))

(defun
   (dired %dir
      (save-window-excursion
         (setq %dir (arg 1 (concat ": dired on directory? [" Dired-dir "] ")))
         (if (= %dir "")
	    (setq %dir Dired-dir)
	 )
	 (setq %dir (expand-file-name %dir))
	 (if (= "/" (substr %dir -1 1))
            (setq %dir (substr %dir 1 -1))
         )
	 (setq Dired-dir %dir)
	 (pop-to-buffer "dired")
	 (setq mode-line-format (concat "  Editing  directory:  " Dired-dir
                                 		    "      %M   %[%p%]")
	 )
	 (erase-buffer)
         (set-mark)
	 (message "Getting contents of " Dired-dir)
	 (sit-for 0)
	 (fast-filter-region (concat "ls -l " Dired-dir))
         (beginning-of-file)
	 (if (looking-at "total")
	    (progn
	       (kill-to-end-of-line)
               (kill-to-end-of-line)
	       (beginning-of-file)
	       (re-replace-string "^." " &")
               (beginning-of-file)
	    )
 	    ; Assumption: The last line is the shell's error message.
	    (progn 
	       (end-of-file)
               (error-message (region-to-string))
	    )
	 )
         (message "Type ^C to exit, ? for help")
         (save-excursion (recursive-edit))
         (&dired-done)
         (novalue)
      )
   )

   (&dired-Mark-file-deleted
      (if (= 0 (buffer-size))
	 (error-message "dired already done!")
	 (progn
            (beginning-of-line)
            (if (looking-at " d")
               (error-message "You can't delete a directory with dired")
	    )
         (delete-next-character)
         (insert-string "D")
         (next-line)
         (beginning-of-line)
	 )
      )
   )

   (&dired-UnMark-file-deleted
      (if (= 0 (buffer-size))
	 (error-message "dired already done!")
	 (progn
	    (beginning-of-line)
	    (delete-next-character)
            (insert-string " ")
	    (next-line)
	    (beginning-of-line)
	 )
      )
   )
    
   (&dired-summary
      (message "<d>el, <u>ndel, <q>uit, <r>emove, <e>xamine, "
                             "<v>isit, <n>ext, <p>rev, <?>-help")
   )

   (&dired-backup-unmark
      (if (= 0 (buffer-size))
         (error-message "dired already done!")
          (! (bobp))
         (previous-line)
      )
      (beginning-of-line)
      (delete-next-character)
      (insert-string " ")
      (beginning-of-line)
   )

   (&dired-examine ans
      (save-excursion
        (error-occurred
           (visit-file (&dired-get-fname))
           (message "Type ^C to return to DIRED")
           (recursive-edit)
        )
      )
   )


   (&dired-remove
      (beginning-of-line)
      (kill-to-end-of-line)
      (kill-to-end-of-line)
   )
    
   (&dired-get-fname
      (save-excursion
         (beginning-of-line)
         (goto-character (+ (dot) 46))
         (set-mark)
         (end-of-line)
         (concat Dired-dir "/" (region-to-string))
      )
   )
    
   (&dired-done ans
      (beginning-of-file)
      ; This might be trying to replace all lines beginning with a space.
      (re-replace-string "^ .*\n" "")
      (if (!= 0 (buffer-size))
         (progn
	    (message "? [y-go through marked files;"
                     " e-don't delete, exit; Anything else return]")
            (setq ans (get-tty-character))
            (if (c= ans 'e')
	       (progn
	          (message "Really exit without deleting? "
                           "[y=yes, anything else continue dired]")
                  (setq ans (get-tty-character))
	          (if (c= ans 'y')
 	             (error-message "Aborted.")
		  )
	       )
	        (c= ans 'y')
               (progn
	          (while (! (eobp))
                     (if (= (following-char) 'D')
		        (progn thisfile ans
		           (setq thisfile (&dired-get-fname))
		           (message "Delete " thisfile "?")
		           (setq ans (get-tty-character))
                           (if (c= ans 'y')
			      (if (unlink-file thisfile)
			         (progn
			            (message "Couldn't delete it!")
			            (sit-for 2)
				 )
			      )
			   )
			)
		     )
                     (next-line)
		  )
	       )
	    )
            (error-message "Aborted.")
	 )
         (message "exited dired")
      )
   )
)
    
   (progn loop
      (save-excursion
         (temp-use-buffer "dired")
         (define-keymap "&dired-keymap")
         (use-local-map "&dired-keymap")
	 (local-bind-to-key "&dired-summary" "?")
         (local-bind-to-key "&dired-Mark-file-deleted" "d")
         (local-bind-to-key "&dired-Mark-file-deleted" "D")
         (local-bind-to-key "&dired-Mark-file-deleted" "\^D")
         (local-bind-to-key "&dired-backup-unmark" "\0177")
         (local-bind-to-key "previous-line" "\^H")
         (local-bind-to-key "previous-line" "p")
         (local-bind-to-key "previous-line" "P")
         (local-bind-to-key "previous-line" "\^P")
         (local-bind-to-key "next-line" "n")
         (local-bind-to-key "next-line" "N")
         (local-bind-to-key "next-line" "\^N")
         (local-bind-to-key "next-line" 13)
         (local-bind-to-key "next-line" 10)
         (local-bind-to-key "next-line" " ")
         (local-bind-to-key "argument-prefix" "\^U")
         (local-bind-to-key "previous-window" "\^Xp")
         (local-bind-to-key "previous-window" "\^XP")
         (local-bind-to-key "next-window" "\^Xn")
         (local-bind-to-key "next-window" "\^XN")
         (local-bind-to-key "delete-window" "\^Xd")
         (local-bind-to-key "delete-window" "\^XD")
         (local-bind-to-key "delete-other-windows" "\^X1")
         (local-bind-to-key "&dired-examine" "\^X\^V")
         (local-bind-to-key "next-page" "\^V")
         (local-bind-to-key "previous-page" "\ev")
         (local-bind-to-key "previous-page" "\eV")
         (local-bind-to-key "beginning-of-file" "\e<")
         (local-bind-to-key "end-of-file" "\e>")
         (local-bind-to-key "&dired-UnMark-file-deleted" "u")
         (local-bind-to-key "&dired-UnMark-file-deleted" "U")
         (local-bind-to-key "exit-emacs" "\^C")
         (local-bind-to-key "exit-emacs" "q")
         (local-bind-to-key "exit-emacs" "Q")
         (local-bind-to-key "&dired-examine" "e")
         (local-bind-to-key "&dired-examine" "E")
         (local-bind-to-key "&dired-examine" "v")
         (local-bind-to-key "&dired-examine" "V")
         (local-bind-to-key "&dired-remove" "r")
         (local-bind-to-key "&dired-remove" "R")
      )
   )
