;---------------------------------------------------------------------------
;NAME
;   elec-c.ml -- major mode for entering and editing C text
; 
;SYNOPSIS
;   elec-c      -- turn on electric-c mode
;   comment     -- toggle comment submode manually
;   verbatim    -- turn off all abbreviations and special handling
;   voltage	-- toggle high-voltage expansion of C constructs
;
;KEY BINDINGS
;   See DESCRIPTION.
;
;DESCRIPTION
;	Once the package is  loaded,  local  key  bindings  and  local 
;	abbreviations are initialized within a buffer by executing the
;	function elec-c-mode. The most convenient way to do this is by
;	including in your .emacs_pro file the commands:
;		(autoload "elec-c-mode" "elec-c.ml")
;		(auto-execute "elec-c-mode" "*.c")
;
; The main functions provided by this package are:
; 
;	1.  Expansion of C  control constructs:   if,  while, do, switch,
;	for.  When the user  is at  the end  of a  line and  types one of
;	these C keywords immediately followed by  a space,  a skeleton of
;	the  corresponding  control  construct is  created.   The user is
;	placed in recursive  edit mode  to fill  in ()  or {}.   When the
;	filling in is complete, the user types  ^C to  exit the recursive
;	edit.  This should be done while the  cursor is  still within the
;	() or {}.  
;
;	NOTE:  the user is also placed in  recursive edit  mode to insert
;	the constant  expression following  'case' in  a switch statement
;	and should use ^C to  move from  the case  constant expression to
;	the body of the case.  
;
;	For example, when the  user  types  a  space  following "if"  the 
;	following appears in the buffer:
; 
;	        if ()
;	        {
;	        }
; 
;	and the cursor is between the parentheses.  The  user then enters
;	some boolean expression within the parens,  followed by  ^C.  The
;	^C will exit the recursive edit and cause  the cursor  to move to
;	the body of the if  construct.   The user  then enters additional
;	statements,  the last  of which  should be  followed by  ^C.  The
;	cursor will be  left on  a new  line immediately  below the right
;	brace, "}".  If the user was on  a blank  line when  he typed ^C,
;	that line will have been deleted.  
;	
;	The  variable  &elec-c-indent-style  can  be used  to control the
;	indentation style of the blocks generated by this feature.  
;
;	The styles available are:
;
;	1 = Kernel style:
;	        <if-while-for> (<control-clause>) {
;	                <body>
;	        }
;
;	2 = Allman style:
;	        <if-while-for> (<control-clause>)
;	        {
;	                <body>
;	        }
;
;	3 = Whitesmiths style:
;	        <if-while-for> (<control-clause>)
;	                {
;	                <body>
;	                }
;
;	Else clauses are indented like if-while-fors except that the
;	control clause is absent. The default value  is 2,  and  the
;	examples above are in Allman style;  the  value  can  easily
;	be set to 1 or 3 in the user's profile if desired.
;
;	If the variable &elec-c-strip-braces is on (which is the  default)
;	blocks containing a single statement will get the braces and extra
;	indent stripped off on exit.
;
;	C-keyword expansion can be toggled with the (voltage) command.
;	It can be initially enabled by a setq-default of the variable
;	&elec-c-high-voltage to 1 before loading the package.
; 
; 2. Various characters in C almost always come in pairs: {}, (), [].
;	When the user types the first, he gets the second
;	as well, with special formatting done for {}.  You can always
;	quote (with ^Q) the left "paren" to avoid the expansion.
;
; 3. Semicolon and linefeed try to put the user where it is likely he
;	wants to be. 'elec-linefeed' is actually rebound from 'newline-
;	and-indent'.
;
; 4. Comma expands to comma followed by space, saving a nuisance keystroke.
;
; 5. The slash and asterisk are bound to  special  functions that try to
;	do the right thing as far as comments are concerned.   When  you
;	type /* you will be put into commment mode, in which the control
;	constructs are not expanded,  and  when you subsequently type */
;	comment mode is terminated.
;
; 6. Comments are automatically justified.   You  get  BLOCKED  comments
;	when you type the &elec-c-border-char (the default is '-') after
;	the opening "/*" of a comment.  To close a blocked comment, type
;	the  &elec-c-border-char  immediately  after  a  subsequent '*',
;	presumably at the left end of the block.
; 
; 7. Default settings of global variables:
;	&elec-c-indent-width            4       ; indentation of expressions
;	&elec-c-right-margin            75      ; fold comments here
;	&elec-c-border-char             '-'     ; char for block comments
;	&elec-c-border-size             75      ; border width
;	&elec-c-comment-prefix          "  "    ; for non-block comments
;	&elec-c-split-declarations      0       ; justification option
;	&elec-c-indent-style            2       ; Allman style
;	&elec-c-strip-braces            1       ; strip single statements
;	&elec-c-high-voltage		0	; enable keyword expansion
;
; 6. A C program justification function is bound to ESC-j.  It will
;	re-indent a block (bounded by "{}" (at the left margin)).  It's
;	a bit slow; expect to wait.  You will be told when it's done.
;
; 7. #d and #i are defined as abbreviations for #define and #include,
;	respectively.
;
; 8.    There  is a  function called  "comment" that  allows you to
;	toggle comment mode manually (without typing "/*").   This can be
;	used  in  editing  existing comments.   You  have to  be inside a
;	comment when you toggle comment mode ON.  You  can toggle comment
;	mode  OFF by  either re-executing  the comment  function or, more
;	simply, by typing ^C.  
;	
; 9. There is a function called "verbatim", which causes all special
;	elec-c processing to be turned off until the user types ^C.
;
; 10. 	"main" always goes at the left margin and expands as you expect it
;	to.  Typing "argc" within the parameter  list  of "main"  causes
;	automatic insertion of argv and their declarations.
;
; 11. The quotation marks are bound to special functions that attempt to
;	prevent expansions when you are inside quotations.  If you find
;	that expressions are suddenly not expanding as they should, then
;	you have probably typed a quote mark without typing its closing
;	mate.
;
; 12. Some hooks are available for users wishing to customize their c
;	mode for a particular terminal etc. These are:
;	        elec-c-mode-load-hook   -- executes at end of load
;	        elec-c-mode-init-hook   -- executes at end of init
;	        elec-c-mode-exit-hook   -- executes on top-level exit
;
; HISTORY
; {3}	19-Feb-85  Eric S. Raymond (eric) at rabbit1
;	Fixed a bug in main() expansion.
;
; {2}	 3-Oct-84  David Medinets (dm) at unipress
;	The  if  (error-occured  (use-keymap  "xyz"))   then  define,  is
;	annoying  when  stack-trace-on-error  is  set; So  I have defined
;	the keymaps in the initialization section.
;
; 1-Aug-84  Eric S. Raymond at Rabbit Software
;	Added voltage toggle. Suppressed else expansion entirely.
;
; 26-Jul-84  Eric S. Raymond at Rabbit Software
;	Fixed control constructs to expand only at end of line.
;
; 26-Jun-84  Eric S. Raymond. 
;	Modified to use rebind facilities. Fixed bug in semicolon
;	handling. Added comma abbreviation. Added strip-braces code.
;
; 15-Jun-84  Eric S. Raymond.
;	Added Whitesmiths style (ugh!) to the indent-filter's repertoire
;	and taught it to use &elec-c-right-margin.
;
; 01-May-84  Eric S. Raymond.
;	Brought up indent-style feature, added user-definable hooks.
;	Significantly reorganized the internals	around the open-block
;	and edit-block functions, reducing the package size by a bit
;	in spite of the new features.
;
; 03-Apr-83  Duane Williams (dtw) at Carnegie-Mellon University
;	Updated documentation and removed experimental functions in
;	preparation for installation on /usr/local.
;
; 02-Feb-82  Duane Williams (dtw) at Carnegie-Mellon University
;	Created.
;---------------------------------------------------------------------------
(if (! (is-bound &elec-c-split-declarations))
    (setq-default &elec-c-split-declarations 0))

(if (! (is-bound &elec-c-indent-width))
    (setq-default &elec-c-indent-width 4))

(if (! (is-bound &elec-c-right-margin))
    (setq-default &elec-c-right-margin 79))

(if (! (is-bound &elec-c-border-char))
    (setq-default &elec-c-border-char '-'))

(if (! (is-bound &elec-c-border-size))
    (setq-default &elec-c-border-size 75))

(if (! (is-bound &elec-c-comment-prefix))
    (setq-default &elec-c-comment-prefix "  "))

(if (! (is-bound &elec-c-indent-style))
    (setq-default &elec-c-indent-style 2))

(if (! (is-bound &elec-c-strip-braces))
    (setq-default &elec-c-strip-braces 1))

(if (! (is-bound &elec-c-high-voltage))
    (setq-default &elec-c-high-voltage 0))

(setq-default &elec-c-comment 0)            ; PRIVATE VARIABLES
(setq-default &elec-c-box-comment 0)
(setq-default inside-double-quote 0)
(setq-default inside-single-quote 0)
    
; Allow these variables to have different values in each elec-c buffer
(declare-buffer-specific 
    &elec-c-indent-width 
    &elec-c-comment      
    &elec-c-box-comment
    &elec-c-right-margin
    &elec-c-border-char
    &elec-c-border-size
    &elec-c-comment-prefix
    inside-double-quote
    inside-single-quote
)

(defun

  (current-position
    (+ (dot) 0)
  )
    
  ;---------------------------------------------------------------------
  ;inside-c-comment-position
  ; if inside a comment, returns the column of the star
  ; that initiates the comment; otherwise returns zero.
  ;---------------------------------------------------------------------

  (inside-c-comment position star-position
    (setq &elec-c-box-comment 0)
    (setq position (current-position))
    (save-excursion
      (if (error-occurred (search-reverse "/*"))
	 0
	 (progn
	   (setq star-position (+ 1 (current-column)))
	   (if (looking-at (concat "/\\*" 
                                   (char-to-string &elec-c-border-char)
                           )
               )
	   (setq &elec-c-box-comment 1))
	   (if (| (error-occurred (search-forward "*/"))
		  (< position (current-position))
               )
	       star-position 0)
         )
      )
    )
  )
    
  (backward-brace level
    (if (error-occurred (search-reverse "}"))
       (error-message "Can't find end of block!")
    )
    (setq level 1)
    (while (> level 0)
      (re-search-reverse "[{}]")
      (if (= (following-char) '{')
         (setq level (- level 1))
	 (setq level (+ level 1))
      )
    )
    (dot)
  )
    
    (forward-brace level
	(if (error-occurred (search-forward "{"))
	    (error-message "Can't find beginning of block!"))
	(setq level 1)
	(while (> level 0)
	    (re-search-forward "[{}]")
	    (if (= (preceding-char) '{')
		(setq level (+ level 1))
		(setq level (- level 1)))
	)
    )
    
    (expand-abbrev space-position   ; cause abbrev expansion without
	(setq space-position (dot)) ; leaving extra chars in buffer
	(insert-character '#')
	(backward-character)
	(insert-character ' ')
	(save-window-excursion
	    (goto-character space-position)
	    (if (= (following-char) ' ')
		(provide-prefix-argument 2
		    (delete-next-character))
		(delete-next-character)
	    )
	)
    )
    
; 
; leave Emacs
; 
    (exit-elec-c-mode
	(if (= (recursion-depth) 0)
	   (error-occurred (elec-c-mode-exit-hook))
	)
	(exit-emacs)
    )
    
; 
; leave recursive edit
; 
    (pop-recursion
	(expand-abbrev)
	(exit-elec-c-mode)
    )
    
; skip over spaces and tabs
  (skip-white-space
    (while (& (! (eolp))
              (| (= (following-char) ' ') (= (following-char) '       '))
           )
      (forward-character)
    )
  )
    
    (this-line-is-blank             ; true if only white space
	(save-excursion             ; on current line
	    (beginning-of-line)
	    (skip-white-space)
	    (eolp)
	)
    )
    
    (kill-current-line              ; delete current line
	(beginning-of-line)
	(if (eolp)
	    (kill-to-end-of-line)
	    (provide-prefix-argument 2 (kill-to-end-of-line)))
    )

    (beginning-of-block  mymark     ; jump to head of {} block
	(insert-string "}")
	(setq mymark (dot))
	(if (error-occurred (backward-brace))
	    (progn
		(goto-character mymark)
		(delete-previous-character))
	    (save-excursion
		(goto-character mymark)
		(delete-previous-character))
	)
    )
    
    (end-of-block                   ; jump to end of {} block
	(beginning-of-block)
	(forward-brace)
    )
    
    (preceding-word                 ; return preceding word
	(backward-word)
	(set-mark)
	(forward-word)
	(region-to-string)
    )


    ;---------------------------------------------------------------------
    ;open-block
    ;    This function accepts a string containing {} and opens up the
    ;braces in the style indicated by the &elec-c-indent-style variable.
    ;It returns a marker for the position between the braces.
    ;    The second argument is interpreted as a boolean enabling insert
    ;of whitespace before the leading brace.
    ;    If the string contains (), dot is positioned between the parens
    ;when the function exits, otherwise it is at the start of the inserted
    ;string.
    ;---------------------------------------------------------------------

    (open-block                     ; Format braces in C control structure
	brace-body                          ; our local variables
	(insert-string (arg 1))             ; insert structure skeleton
	(beginning-of-line)                 ; want the first (
	(error-occurred
	    (search-forward "("))           ;   go find it
	(save-window-excursion
	    (end-of-line)
	    (search-reverse "}")            ; find the brace body location
	    (setq brace-body (dot))
	    (newline-and-indent)
	    (if (= &elec-c-indent-style 3)  ; Whitesmiths style
		(elec-tab))
	    (previous-line)
	    (search-forward "{")            ; find it
	    (backward-character)
	    (if (arg 2)                     ; if new-block enabled
		(if (= &elec-c-indent-style 1)      ; if kernel style
		    (insert-string " ")             ;   just insert space
		    (progn                          ; else
			(newline-and-indent)        ;   start new block
			(if (= &elec-c-indent-style 3)      ; Whitesmiths
			    (elec-tab))
		    )
		)
	    )
	)
	brace-body                  ; dot is now at control clause if any
    )

    ;---------------------------------------------------------------------
    ;edit-block
    ;    This function assumes that dot is positioned between braces. It
    ;does a recursive-edit of the block and, upon exit, moves to the
    ;position indicated by &elec-c-indent-style for the next construct.
    ;    Its first argument is a mode string for the block. Its second
    ;indicates the character to be searched for to find the end-of-block.
    ;---------------------------------------------------------------------

    (edit-block                             ; edit a {} block
	block-start next-position           ; hold start and end marks
	(setq mode-string (arg 1))          ; announce the kind of block
	(save-window-excursion
	    (search-forward (arg 2))        ; find the end of this block
	    (setq next-position (dot))      ; and mark it
	)
	(if (= &elec-c-indent-style 3)      ; now we go into the block
	    (progn
		(end-of-line)
		(newline-and-indent)        ; Whitesmiths uses last indent
	    )
	    (elec-indent)                   ; other styles add a level
	)
	(recursive-edit)                    ; edit the block text
	(if (this-line-is-blank)            ; if no stuff for block
	    (kill-current-line)             ;   close it up
	)
	(goto-character next-position)      ; go to end of block

	(if (&
		(= (preceding-char) '}')   ; we're at the end of a non-do
		&elec-c-strip-braces	   ; brace-stripping is enabled
		(> (current-column) 2)	   ; they aren't function-body braces
		(one-semicolon
		    (setq block-start (backward-brace))
		    next-position
		)
	    )
	    (strip-braces block-start next-position)
	)
        (progn
	    (newline-and-indent)
	    (if (= &elec-c-indent-style 3)
		(subtract-indent))
	)
    )

    ;---------------------------------------------------------------------
    ;one-semicolon
    ;    Predicate, returns true if block has exactly one semicolon.
    ;The two arguments are start- and end-of-block positions.
    ;---------------------------------------------------------------------

    (one-semicolon
	colons
	(save-excursion
	    (setq colons 0)
	    (goto-character (arg 1))
	    (while (< (dot) (arg 2))
		(if (= (preceding-char) ';')
		    (setq colons (+ colons 1))
		)
		(forward-character)
	    )
	)
	(= colons 1)
    )

    ;---------------------------------------------------------------------
    ;delete-brace
    ;    Delete the following char, and delete the line it's on if what's
    ;left is all whitespace.
    ;---------------------------------------------------------------------

    (delete-brace
	(delete-next-character)
	(while (! (bolp))
	    (backward-character)
	    (if (looking-at "[ \t]") (delete-next-character))
	)
	(if (looking-at "^$") (delete-next-character))
    )

    ;---------------------------------------------------------------------
    ;strip-braces
    ;    Strips braces off a block if it contains but a single statement
    ;The two arguments are start- and end-of-block positions.
    ;---------------------------------------------------------------------

    (strip-braces
	(goto-character (arg 1))
	(delete-brace)
	(goto-character (arg 2))
	(backward-character)
	(delete-brace)
	(backward-character)
    )

    ;---------------------------------------------------------------------
    ;elec-left-brace
    ;    This calls open-block and block-edit to fill in a braces-block at
    ;the current indent level.
    ;---------------------------------------------------------------------

    (elec-left-brace old-mode               ; Takes 0 arguments
	(if (& (eolp) &elec-c-high-voltage)
	    (progn
		(setq old-mode mode-string)           ; Preserve old mode.
		(goto-character (open-block "{}" 0))  ; Open a new block.
		(edit-block "block body" "}")         ; Recursive-edit it.
		(setq mode-string old-mode)           ; Restore old mode.
		0                           ; Don't want sp after abbrev
	    )
	    (insert-character '{')
	)
    )                                       ; End of elec-left-brace

    ;---------------------------------------------------------------------
    ;The following functions help with various delimiters etc.
    ;---------------------------------------------------------------------

    (elec-left-paren                        ; insert paren pair
	(insert-string "()")
	(backward-character)
    )
    
    (elec-left-bracket                      ; insert bracket pair
	(insert-string "[]")
	(backward-character)
    )
    
    ;---------------------------------------------------------------------
    ;elec-right-paren
    ;    Insert a close-paren, flash the previous balancing one.
    ;---------------------------------------------------------------------

    (elec-right-paren                       ; Takes 0 arguments
	line next-position
	(insert-character (last-key-struck))
	(save-excursion
	    (backward-paren)
	    (if (dot-is-visible)
		(progn
		    (setq line "")
		    (sit-for 10))
		(progn
		    (beginning-of-line)
		    (set-mark)
		    (end-of-line)
		    (setq line (region-to-string)))
	    )
	)
	(message line)
    )
    
    (elec-tab tabcount
        (if (= &elec-c-indent-width 0)
           (setq &elec-c-indent-width 0)
        )
	(setq tabcount (/ (current-column) &elec-c-indent-width))
	(insert-character '.')
	(backward-character)
	(delete-white-space)
	(delete-next-character)
	(to-col (+ (* (+ tabcount 1) &elec-c-indent-width) 1))
    )
    
    (elec-function-format old-dot old-size
	(setq old-dot (+ (dot) 0))
	(setq old-size (buffer-size))
	(save-excursion
	    (previous-line)
	    (re-search-forward "^}")
	    (set-mark)
	    (backward-brace)
	    (beginning-of-line)
	    (exchange-dot-and-mark)
	    (end-of-line)
	    (forward-character)
	    (filter-region 
		(concat 
		    "indent " 
		    "-st "
		    "-l" &elec-c-right-margin
		    "-i" &elec-c-indent-width
		    (if  &elec-c-split-declarations " -bc " " -nbc ")
		    (if (= &elec-c-indent-style 2) " -bl")
		    (if (= &elec-c-indent-style 3) " -bw")
		)
	    )
	)
	(goto-character (/ (* (buffer-size) old-dot) old-size))
	(message "Done!")
    )

    ;---------------------------------------------------------------------
    ;add-indent
    ;    Add a level of indenting to the current line
    ;---------------------------------------------------------------------

    (add-indent
	(newline-and-indent)
	(provide-prefix-argument
	    &elec-c-indent-width (insert-character ' '))
    )

    ;---------------------------------------------------------------------
    ;subtract-indent
    ;    Remove a level of indenting from the current line
    ;---------------------------------------------------------------------

    (subtract-indent n
	(setq n &elec-c-indent-width)
	(while (> n 0)
	    (setq n (- n 1))
	    (if (= (preceding-char) "\t")
		(progn
		    (delete-previous-character)
		    (insert-string "       ")       ; 7 spaces
		)
		(delete-previous-character)
	    )
	)
    )

    (elec-indent
	(end-of-line)
	(add-indent)
    )
    
    (elec-linefeed
	(if (looking-at "[\t ]*$")
	    (progn
		(delete-white-space)
		(newline-and-indent)
	    )
	    (add-indent)
	)
    )
    
    (elec-semicolon
	(if (looking-at "[\t ]*$")
	    (progn 
		(delete-white-space)
		(insert-character ';')
		(newline-and-indent)
	    )
	    (insert-character ';')
	)   
    )
    
    ;---------------------------------------------------------------------
    ;elec-main
    ;    Expand to a main (possibly with arguments) and do a block-edit
    ;to fill it.
    ;---------------------------------------------------------------------

  (elec-main next-position old-mode         ; insert main construct
    (if (& (eolp) &elec-c-high-voltage)
      (progn
        (setq old-mode mode-string)
	(beginning-of-line)                 ; put main at margin
	(delete-white-space)
	(end-of-line)
	(setq next-position (open-block " (){}" 1))
	(setq mode-string "main arguments")
	(use-abbrev-table "elec-c-main-abbrevs")
	(local-rebind "pop-recursion" "exit-emacs" "\^C")
	(recursive-edit)
	(local-rebind "exit-elec-c-mode" "pop-recursion" "\^C")
	(use-abbrev-table "elec-c-abbrevs")
	(goto-character next-position)
	(edit-block "main body" "}")
	(setq mode-string old-mode)
	0
      )
      1
    )
  )
    
    ;---------------------------------------------------------------------
    ;elec-argc
    ;    Expand to arc/argv declarations for the main().
    ;---------------------------------------------------------------------

    (elec-argc                              ; auto declare argc/argv
	(search-forward ")")
	(backward-character)
	(insert-string ", argv")
	(search-forward "{")
	(backward-character)
	(insert-string "int\targc;")
	(newline)
	(insert-string "char\t*argv[];")
	(newline)
	0
    )
    
  (elec-if next-position old-mode         ; insert if construct
    (if (& (eolp) &elec-c-high-voltage)
      (progn
        (setq old-mode mode-string)
	(setq next-position (open-block " (){}" 1))
	(setq mode-string "if condition")
	(recursive-edit)
	(goto-character next-position)
	(edit-block "if-body" "}")
	(setq mode-string old-mode)
	0
      )
      1
    )
  )
    
;    (elec-else old-mode                     ; insert else construct
;	(if (& (eolp) &elec-c-high-voltage)
;	    (progn
;		(setq old-mode mode-string)
;		(goto-character (open-block "{}" 1))
;		(edit-block "else body" "}")
;		(setq mode-string old-mode)
;		0
;	    )
;	    1
;	)
;    )
    
    (elec-for next-position old-mode        ; insert for construct
	(if (& (eolp) &elec-c-high-voltage)
	    (progn
		(setq old-mode mode-string)
		(setq next-position (open-block " (;;){}" 1))
		(setq mode-string "for control")
		(recursive-edit)
		(goto-character next-position)
		(edit-block "for body" "}")
		(setq mode-string old-mode)
		0
	    )
	    1
	)
    )
    
    (elec-while next-position old-mode      ; insert while construct
	(if (& (eolp) &elec-c-high-voltage)
	    (progn
		(setq old-mode mode-string)
		(setq next-position (open-block " (){}" 1))
		(setq mode-string "while condition")
		(recursive-edit)
		(goto-character next-position)
		(edit-block "while body" "}")
		(setq mode-string old-mode)
		0
	    )
	    1
	)
    )
    
    (elec-do next-position old-mode         ; insert do construct
	(if (& (eolp) &elec-c-high-voltage)
	    (progn
		(setq old-mode mode-string)
		(setq next-position (open-block " {} while ();" 0))
		(setq mode-string "do condition")
		(recursive-edit)
		(goto-character next-position)
		(edit-block "do body" ";")
		(setq mode-string old-mode)
		0
	    )
	    1
	)
    )

    (elec-switch next-position old-mode     ; insert switch construct
	(if (& (eolp) &elec-c-high-voltage)
	    (progn
		(setq old-mode mode-string)
		(setq next-position (open-block " (){}" 1))
		(setq mode-string "switch condition")
		(recursive-edit)
		(goto-character next-position)
		(save-window-excursion 
		    (search-forward "}")
		    (setq next-position (dot))
		)
		(newline-and-indent)
		(elec-case-loop)
		(goto-character next-position)
		(newline-and-indent)
		(setq mode-string old-mode)
		0
	    )
	    1
	)
    )
    
    (elec-case next-position old-mode       ; single case statment
	(if (& (eolp) &elec-c-high-voltage)
	    (progn
		(setq old-mode mode-string)
		(insert-string " :break;")          ; expand case statement
		(search-reverse "b")
		(add-indent)
		(search-reverse ":")                ; goto case condition
		(save-window-excursion              ; prepare next position
		    (elec-indent)
		    (setq next-position (dot))
		)
		(setq mode-string "case condition")
		(recursive-edit)                    ; edit case condition
		(goto-character next-position)
		(save-window-excursion 
		    (search-forward ";")
		    (setq next-position (dot))
		)
		(setq mode-string "case body")
		(recursive-edit)
		(if (this-line-is-blank)
		    (kill-current-line)
		)
		(goto-character next-position)
		(setq mode-string old-mode)
		0
	    )
	    1
	)
    )
    
    (elec-case-loop case-col next-position old-mode
	(setq old-mode mode-string)
	(setq case-col (current-indent))    ; save case column
	(insert-string "case :break;")      ; expand case statement
	(search-reverse "b")
	(add-indent)
	(search-reverse ":")                ; goto case condition
	(save-window-excursion              ; prepare next position
	    (elec-indent)
	    (setq next-position (dot))
	)
	(setq mode-string "case condition")
	(recursive-edit)                    ; edit case condition
	(while (!= (preceding-char) ' ')
	    (goto-character next-position)  ; goto case body
	    (save-window-excursion          ; prepare next position
		(search-forward ";")
		(newline)
		(to-col case-col)
		(setq next-position (dot))
	    )
	    (setq mode-string "case body")
	    (recursive-edit)                ; edit body of case
	    (if (this-line-is-blank)
		(kill-current-line)
	    )
	    (goto-character next-position)  ; find end of case
	    (insert-string "case :break;")  ; setup next case
	    (search-reverse "b")
	    (add-indent)
	    (search-reverse ":")            ; goto case condition
	    (save-window-excursion          ; prepare next position
		(elec-indent)
		(setq next-position (dot))
	    )
	    (setq mode-string "case condition")
	    (recursive-edit)                ; edit case condition
	)
	(delete-previous-word)              ; setup for default
	(insert-string "default")
	(next-line)
	(setq mode-string "default")
	(recursive-edit)
	(if (this-line-is-blank)
	    (kill-current-line)
	)
	(setq mode-string old-mode)
	0
    )
    
    (elec-default next-position old-mode    ; case default
	(if (& (eolp) &elec-c-high-voltage)
	    (progn
		(setq old-mode mode-string)
		(insert-string ":break;")
		(search-reverse "b")
		(add-indent)
		(previous-line)
		(elec-indent)
		(save-window-excursion 
		    (search-forward ";")
		    (setq next-position (dot))
		)
		(recursive-edit)
		(goto-character next-position)
		(setq mode-string old-mode)
		0
	    )
	    1
	)	
    )
    
    (setup-for-code
	(setq &elec-c-comment 0)
	(setq left-margin 0)
	(setq right-margin 0)
	(setq prefix-string "")
	(use-local-map "elec-c-keymap")
	(setq abbrev-mode 1)
    )
    
    (elec-slash                             ; watch for close comment
	(if (= (preceding-char) '*')
	    (progn
		(insert-character '/')
		(exit-elec-c-mode)
	    )
	    (insert-character '/')
	)
    )

    (verbatim old-mode next-position
	(setq old-mode mode-string)
	(setq next-position (dot))
	(setq left-margin 0)
	(setq right-margin 0)
;	(setq prefix-string "")
	(setq abbrev-mode 0)
        (use-local-map "elec-c-verbatim-keymap")
	(setq mode-string "verbatim")
	(message "Type ^C to exit verbatim mode")
	(recursive-edit)
	(goto-character next-position)
	(setq mode-string old-mode)
	(if &elec-c-comment
	    (setup-for-comment)
	    (setup-for-code)
	)
    )

    (voltage
	(setq &elec-c-high-voltage (! &elec-c-high-voltage))
    )

    (setup-for-comment
	(if (!= (preceding-char) '/')
	    (if (=  0 (setq left-margin (inside-c-comment)))
		(setq left-margin (current-column))
	    )
	    (setq left-margin (current-column))
	)
	(setq right-margin &elec-c-right-margin)
	(if &elec-c-box-comment             ; set by inside-c-comment
	    (setq prefix-string "* ")
	    (setq prefix-string &elec-c-comment-prefix)
	)
	(use-local-map "elec-c-comment-keymap")
	(setq abbrev-mode 0)
	(setq &elec-c-comment 1)
    )
	
    (elec-star old-mode                     ; watch for open comment
	(if (= (preceding-char) '/')
	    (progn
		(insert-character '*')
		(comment)
	    )
	    (insert-character '*')
	)
    )
	
    (elec-border old-prefix                 ; create comment border
	(if (& &elec-c-comment 
		(= (preceding-char) '*'))
	    (progn 
		(setq right-margin 10000)
		(provide-prefix-argument &elec-c-border-size
		    (insert-character &elec-c-border-char))
		(newline-and-indent)
		(setq right-margin &elec-c-right-margin)
		(insert-character '.')
		(backward-character)
		(previous-line)
		(if (= (following-char) '/')
		    (progn
			(next-line)
			(delete-next-character)
			(insert-string " * ")
			(setq old-prefix prefix-string)
			(setq prefix-string "* "))
		    (= (following-char) '*')
		    (progn
			(setq prefix-string old-prefix)
			(next-line)
			(delete-next-character)
			(insert-string "*/")
			(newline)
			(exit-elec-c-mode))
		)
	    )
	    (progn
		(insert-character &elec-c-border-char))
	)
    )
	
    (comment old-mode               ; toggle comment mode manually
	(if &elec-c-comment
	    (exit-elec-c-mode)
	    (progn 
		(setq old-mode mode-string)
		(setq mode-string "comment")
		(setup-for-comment)
		(recursive-edit)
		(setup-for-code)
		(setq mode-string old-mode)
		(novalue)
	    )
	)
    )
	
    (elec-doublequote       	; guard quotes against abbrev expansion
	(insert-string """")
	(if (! inside-single-quote)
	    (if inside-double-quote
		(progn
		    (setq inside-double-quote 0)
		    (setq abbrev-mode 1))
		(progn
		    (setq inside-double-quote 1)
		    (setq abbrev-mode 0))
	    )
	)
    )

    (elec-comma
	 (insert-character ',')
	 (insert-character ' ')
    )
	
    (elec-singlequote       ; guard quotes against abbrev expansion
	(insert-string "'")
	(if (! inside-double-quote)
	    (if inside-single-quote
		(progn
		    (setq inside-single-quote 0)
		    (setq abbrev-mode 1))
		(progn
		    (setq inside-single-quote 1)
		    (setq abbrev-mode 0))
	    )
	)
    )               
	
    (elec-c-mode                            ; initialize elec-c mode
	(use-local-map "elec-c-keymap")
	(use-abbrev-table "elec-c-abbrevs")
	(setq mode-string "elec-c")
	(use-syntax-table "elec-c")
        (error-occurred (elec-c-mode-init-hook))
    )
)

;---------------------------------------------------------------------
;Initialization
;    Here is the main line of the code executed at load time by
;the elec-c-mode package.
;---------------------------------------------------------------------

(save-excursion
  (temp-use-buffer "elec-c buffer")
  (setq needs-checkpointing 0)
    
  (define-keymap "elec-c-keymap")
  (define-keymap "elec-c-verbatim-keymap")
  (define-keymap "elec-c-comment-keymap")

  (use-local-map "elec-c-keymap")

  (local-rebind "elec-linefeed"        "newline-and-indent" "\^j")
  (local-rebind "exit-elec-c-mode"     "exit-emacs"         "\^c")
  (local-rebind "elec-function-format" "indent-C-procedure" "\ej")
  (local-rebind "backward-brace"       "backward-sentence"  "\ea")
  (local-rebind "forward-brace"        "forward-sentence"   "\ee")

  (local-bind-to-key "backward-brace"    "\e{")
  (local-bind-to-key "forward-brace"     "\e}")
  (local-bind-to-key "elec-left-brace"   "{")
  (local-bind-to-key "elec-left-bracket" "[")
  (local-bind-to-key "elec-left-paren"   "(")
  (local-bind-to-key "elec-doublequote"  """")
  (local-bind-to-key "elec-comma"        ",")
  (local-bind-to-key "elec-singlequote"  "'")
  (local-bind-to-key "elec-right-paren"  "}")
  (local-bind-to-key "elec-right-paren"  ")")
  (local-bind-to-key "elec-right-paren"  "]")
  (local-bind-to-key "elec-semicolon"    ";")
  (local-bind-to-key "elec-star"         "*")
  (local-bind-to-key "elec-tab"          "\t")
    
  (use-local-map "elec-c-comment-keymap")
  (local-bind-to-key "elec-slash" '/')
  (local-bind-to-key "elec-border" &elec-c-border-char)
    
  (use-local-map "elec-c-verbatim-keymap")

  (use-abbrev-table "elec-c-abbrevs")
  (define-local-abbrev "#d" "#define")
  (define-local-abbrev "#i" "#include")
  (define-hooked-local-abbrev "if"      "if"      "elec-if")
; (define-hooked-local-abbrev "else"    "else"    "elec-else")
  (define-hooked-local-abbrev "for"     "for"     "elec-for")
  (define-hooked-local-abbrev "while"   "while"   "elec-while")
  (define-hooked-local-abbrev "do"      "do"      "elec-do")
  (define-hooked-local-abbrev "switch"  "switch"  "elec-switch")
  (define-hooked-local-abbrev "case"    "case"    "elec-case")
  (define-hooked-local-abbrev "default" "default" "elec-default")
  (define-hooked-local-abbrev "main"    "main"    "elec-main")

  (use-abbrev-table "elec-c-main-abbrevs")
  (define-hooked-local-abbrev "argc" "argc" "elec-argc")

  (use-syntax-table "elec-c")
  ; make underscore part of a word 
  (modify-syntax-entry "w    _")
  (modify-syntax-entry "w    #")
  ; add all the various types of C parens.
  (modify-syntax-entry "(}   {")
  (modify-syntax-entry "(]   [")
  (modify-syntax-entry "()   (")
  (modify-syntax-entry "){   }")
  (modify-syntax-entry ")[   ]")
  (modify-syntax-entry ")(   )")
    
  (modify-syntax-entry "     """)  ; else forward/backward paren fails
  (modify-syntax-entry "     '")   ; else forward/backward paren fails
  (modify-syntax-entry "  { */")   ; begin comment
  (modify-syntax-entry "   }/*")   ; end comment
  (modify-syntax-entry "\\    \\") ; prefix char
  (error-occurred (elec-c-mode-load-hook))
  (novalue)
)

; elec-c.ml ends here ------------------------------------------------------
