;-----------------------------------------------------------------------------
;NAME
;   elec-lisp.ml -- major mode for editing Lisp and MLisp
;
;SYNOPSIS
;   elec-lisp	-- invoke elec-lisp mode
;   elec-mlisp	-- invoke elec-mlisp-mode
;
;KEY BINDINGS
;   semi-elec-lisp					;
;   paren-pause						)
;   forward-paren		forward-sentence	ESC-)
;   backward-paren		backward-sentence	ESC-(
;   forward-paren					ESC-)
;   backward-paren					ESC-(
;   indent-lisp-function	indent-C-procedure	ESC-j
;   nl-indent			newline-and-indent	\n
;   re-indent-line					ESC-i
;   zap-defun						^X^L	; in lisp
;   execute-mlisp-buffer				^X^L	; in mlisp
;
;DESCRIPTION
;   This is the mode for editing Lisp and MLisp source. It does appropriate
;indenting things with semicolon and the newline-and-indent key.
; 
;   All key-sequences pointing to the indent-C-procedure function (normally
;ESC-j) are rebound to a function that re-indents lisp or mlisp code. ESC-i
;is rebound to a function that fixes the indent of the current line.
;
;   The exact style is controlled by the variable &hanging-indent. If false,
;the mode will simply indent every 4 spaces (using tabs where possible). If
;true (the default) each level of braces will be indented to hang directly
;underneath the second token in its enclosing statement.
;
;   Functions to move to the beginning or end of the current parenthesis
;level are also provided.
;
;   ^X^L acts differently in lisp and mlisp. In lisp it looks for a function
;definition starting on the current line and passes it to a lisp process for
;execution; in mlisp it compiles the entire current buffer.
;
;HISTORY
; {4}	18-Feb-85  Eric S. Raymond (eric) at rabbit1
;	Added (execute-and-notify) so I'd  know when  MLisp compiles have
;	finished.  
;
; {3}	 4-Feb-85  David Medinets (dm) at unipress
;	moved the definitions of the local keymaps to an  init section at
;	the  end  of  a  file.    The  repeated definition  of the keymap
;	everytime that elec-lisp and elec-mlisp modes were invoked caused
;	the old keymaps to disappear.
;
; {2}	31-Dec-84  David Medinets (dm) at unipress
;	I  renamed  electric-lisp  and  electric-mlisp  to  elec-lisp and
;	elec-mlisp for compatibilty.   I also  made both  modes use their
;	own local keymap.
;
; {1}	 3-Oct-84  David Medinets (dm) at unipress
;	Modifyed the syntax table for  lisp and mlisp mode  so that  it 
;	knows what the correct begin, cont, and end comment characters are.
;
;   2-Aug-84  Eric S. Raymond at Rabbit Software
;	Added the &hanging-indent switch, fixed a bug, added mlisp ^X^L.
; 
;   4-Jul-84  Eric S. Raymond at Rabbit Software
;	Replaced many local-bind-to-keys with local-rebinds, added this
;	documentation header. Merged in experimental MLisp features.
;-----------------------------------------------------------------------------
(if (! (is-bound &hanging-indent))
       (setq-default &hanging-indent 1))

(if (! (is-bound &auto-right-paren))
       (setq-default &auto-right-paren 0))


; This function gets bound to ')' -- it flashes to the corresponding
; '(' and fixes the indentation of the ')'.
(defun
  (paren-pause dot instabs
     (if (eolp)
        (delete-white-space)
     )
     (setq instabs (bolp))
     (setq dot (dot))
     (insert-character ')')
     (if &auto-right-paren
        (nl-indent)   ; insert newline and indent next line.
     )
     (save-excursion
        (backward-paren)
	(if instabs
           (save-excursion descol
	      (setq descol (current-column))
	      (goto-character dot)
	      (to-col descol)
           )           
        )
	(if (dot-is-visible)
	   (sit-for 5)
	   (progn
	      (beginning-of-line)
	      (set-mark)
	      (end-of-line)
	      (message (region-to-string))
           )
        )
     )
  )
)

(defun
    (compute-indent	; compute the depth of the next inner indent
	(if 
	    &hanging-indent
	        (progn			; found a left paren, do fancy stuff
		    lim here
		    (setq lim (+ (current-column) 8))
		    (error-occurred
			(re-search-forward "([ \t]*\\w*[ \t]*"))
		    (if (> (setq here (current-column)) lim)
			(- lim 4)
			here
		    )
		)
	    (+ (current-column) 4)
	)
    )
)

; This function gets bound to linefeed, it inserts a newline and
; properly indents the next line.
(defun
    (nl-indent column
	(save-excursion
	    (error-occurred (backward-balanced-paren-line))
	    (setq column
		(if
		    (bolp)
	    		(if (= (following-char) '(')
			    5
	    		    (current-indent)
			)
		    (compute-indent)
		)
	    )
	)
	(newline)
	(to-col column)
    )
)

; This function repairs the indentation of the current line.
(defun    
    (re-indent-line
	(save-excursion column
	    (beginning-of-line)
	    (delete-white-space)
	    (save-excursion
		(if (= (following-char) ')')
		    (progn
			  (forward-character)
			  (backward-paren)
			  (setq column (current-column))
		    )
		    (| (looking-at "else") (looking-at "then"))
		    (progn
			  (insert-character ')')
			  (set-mark)
			  (backward-paren)
			  (setq column (+ (current-column) 1))
			  (exchange-dot-and-mark)
			  (delete-previous-character)
		    )
		    (progn
			  (backward-character)
			  (backward-balanced-paren-line)
			  (setq column
				(if (bolp)
				    (if (| (looking-at "[ \t]*else")
					   (looking-at "[ \t]*then"))
					(+ (current-indent) 3)
					(current-indent)
				    )
				    (compute-indent)
    				)
			  )
			  (if (< column 5)
			      (setq column 5))
		    )
		)
	    )
	    (to-col column)
	)
    )
)

; this function fixes up the indentation of
; an entire lisp function: (defXX to )
(defun
    (indent-lisp-function
	(save-excursion
	    (if (error-occurred (end-of-line) (re-search-reverse "^(def"))
		(error-message "Can't find function"))
	    (set-mark)
	    (forward-paren)
	    (exchange-dot-and-mark)
	    (narrow-region)
	    (error-occurred (re-replace-string "elseif\n[ \t]*" "elseif "))
	    (error-occurred (re-replace-string "\n[\t ]*then" " then"))
	    (widen-region)
	    (delete-white-space)
	    (beginning-of-line)
	    (next-line)
	    (while (& (! (eobp)) (<= (dot) (mark)))
		   (re-indent-line)
		   (next-line)
	    )
	)
	(message "Done!")
    )
)

(defun
    (semi-elec-lisp
	 (if (eolp)
	     (progn (if (! (bolp)) (to-col comment-column))
		    (setq left-margin comment-column)
		    (setq right-margin 77)
		    (setq prefix-string "; ")
		    (insert-string "; "))
	     (insert-character ';'))
    )
)

(declare-buffer-specific shell-prompt-template)

(defun    
  (elec-lisp-mode		; elec-lisp mode initialization
    (use-local-map "elec-lisp")
    (setq shell-prompt-template "[0-9][0-9]*\. *")
    (local-bind-to-key "semi-elec-lisp" ";")
    (local-bind-to-key "paren-pause"    ")")
    (local-rebind "forward-paren" "forward-sentence"    "\e)")
    (local-rebind "backward-paren" "backward-sentence"  "\e(")
    (local-bind-to-key "forward-paren" "\e)")
    (local-bind-to-key "backward-paren" "\e(")
    (local-rebind "indent-lisp-function" "indent-C-procedure" "\ej")
    (local-rebind "nl-indent" "newline-and-indent" "\n")
    (local-bind-to-key "re-indent-line" "\ei")
    (local-bind-to-key "zap-defun" "\^X\^L")
    (setq mode-string "lisp")
    (use-abbrev-table "lisp")
    (use-syntax-table "lisp")
    (error-occurred (elec-lisp-mode-hook))
    (novalue)
  )

    ; this exists so the paren following a defun won't insist on seeing
    ;	a valid command name.
    (ml-paren func
	(insert-character '(')
	(insert-string (setq func (get-tty-command "  (")))
	(insert-character ' ')
	(if (= func "defun") (local-bind-to-key "defun-paren" "("))
	(novalue))

    (defun-paren
	(insert-character '(')
	(local-bind-to-key "ml-paren" "("))

    (execute-and-notify			; {4}
	(execute-mlisp-buffer)
	(message "Done.")
	(sit-for 0)
    )

  (elec-mlisp-mode	; elec-mlisp mode initialization
    (use-local-map "elec-mlisp")
    (local-bind-to-key "ml-paren" "(")
    (local-bind-to-key "expand-mlisp-variable" "\^\")
    (local-bind-to-key "semi-elec-lisp"        ";")
    (local-bind-to-key "paren-pause"    ")")
    (local-bind-to-key "forward-paren" "\e)")
    (local-bind-to-key "backward-paren" "\e(")
    (local-rebind "forward-paren" "forward-sentence"   "\e)")
    (local-rebind "backward-paren" "backward-sentence" "\e(")
    (local-rebind "indent-lisp-function" "indent-C-procedure" "\ej")
    (local-rebind "nl-indent" "newline-and-indent" "\n")
    (local-bind-to-key "re-indent-line" "\ei")
    (local-bind-to-key "execute-and-notify" "\^X\^L")	; {4}
    (setq mode-string "mlisp")
    (use-abbrev-table "mlisp")
    (use-syntax-table "mlisp")
    (error-occurred (elec-mlisp-mode-hook))
    (novalue)
 )

; take the current "(defXX" to ")" region and
; stuff it as input to the "lisp" process
; then whip into the lisp buffer to interact.

    (zap-defun
      (save-excursion
        (end-of-line)
        (search-reverse "(def")
        (set-mark)
        (forward-paren)
        (end-of-line)
        (forward-character)
        (region-to-process "lisp")
      )
      (pop-to-buffer "lisp")
      (end-of-file)
    )
)

; 
; This is the init section of stuff that is only done once.
; 
(use-syntax-table "lisp")
(modify-syntax-entry "()   (")
(modify-syntax-entry ")(   )")
(modify-syntax-entry """    |")
(modify-syntax-entry """    """)
(modify-syntax-entry "\\    \\")
(modify-syntax-entry "  {  ;")                           ;  {1}
(modify-syntax-entry "   } \^j")                         ;  {1} 
(modify-syntax-entry "w    -+!$%^&=_~:/?*<>")
(use-syntax-table "mlisp")
(modify-syntax-entry "()   (")
(modify-syntax-entry ")(   )")
(modify-syntax-entry "\"    '")
(modify-syntax-entry "\"    \"")
(modify-syntax-entry "\\    \\")
(modify-syntax-entry "  {  ;")                           ;  {1}
(modify-syntax-entry "   } \^j")                         ;  {1} 
(modify-syntax-entry "w    -+!$%^&=_~:/?*|a-zA-Z0-9<>")	 	

(define-keymap "elec-mlisp")
(define-keymap "elec-lisp")

(novalue)

; elec-lisp.ml ends here ----------------------------------------------------
