;---------------------------------------------------------------------------
; execute.ml
;
;SYNOPSIS
;   execute-command		-- do a command, pipe output to given buffer
;   kill-command-execution	-- interrupt an executing asynchronous cmd
;   next-command-error		-- go to next error in output list
;
;KEYBINDINGS
; execute-command
; kill-command-execution	^X^K
; next-command-error		^X^N
;
;DESCRIPTION
;  Executes a system command and collects the output in a buffer so that
;  the next-error facility can be used to visit the site of each error.
;  The error messages are identified using the error message template.
;  The command will be executed synchronously, using filter-region,
;  if the global variable use-async-processes is 0.  Otherwise
;  the command will be executed by an asynchronous process.
;  More than one asynchronous system command can be extant at any given
;  time.  Asynchronous processes are not currently available under
;  System III and System V;  use-async-processes is always zero for
;  those systems.
;
;  Although the execute-command function can be invoked interactively,
;  it is really intended to be used by other MLisp packages, such as
;  grep and compile.  This package is meant to provide a consistent,
;  system-independent interface for such packages, so that they do
;  not have to know whether the system supports asynchronous processes,
;  and do not have to worry about updating the mode line, etc.  It also
;  manages many commands simultaneously so that they do not interfere
;  with one another.
;
;  A system command is started by the execute-command function,
;  which accepts a command string that will be passed to a command
;  interpreter, the name of a buffer into which its output will be
;  collected, and an error message format to be used when scanning
;  the output for errors.  The command is said to be associated with
;  the buffer into which its output is (or has been) collected.
;  While the command is executing, the mode-string for that buffer
;  will be set to "Executing".  Synchronous processes block the user
;  from doing anything else until the command has finished.  There is
;  no way as yet to kill synchronous processes; you must wait until
;  they finish of their own accord.  Asynchronous commands run in
;  background, so that the user can do other things while they are
;  running.  Their associated buffer need not be visible in a window.
;  An asynchronous commands can be killed by switching to its associated
;  buffer and executing the kill-command-execution function.
; 
;  When a command exits or is killed, its buffer's mode-string is set
;  to "Done".  NOTE: Because process exit hooks have not yet been
;  fully implemented in Emacs, asynchronous commands are not marked as having
;  exited until the first "next-command-error" command is issued for that
;  buffer.  Emacs does notice when a command exits, and prints a message,
;  but it does not know which one exited and so cannot mark it as such
;  automatically.
;
;  If the command generated a list of messages that contain file names
;  and line numbers, such as compiler error messages, the next-command-error
;  function can be used to visit the file/line indicated by each error
;  message, one after another.  The next-command-error function checks the
;  current buffer to see if it has a command associated with it.  If it does
;  not, it will pop to the last buffer that was associated with a command.
;  Then it calls next-error to visit the line in the file indicated by the
;  error message.
;
;  The user may edit the command output to remove uninteresting error messages
;  before the first call to next-command-error.  The first invocation of
;  next-command-error calls parse-error-messages-in-region, which builds
;  an internal list of errors, does a visit-file for each of the files
;  referenced, and visits the file/line of the first error.  Subsequent
;  calls to next-command-error get the next error on that internal list,
;  which is unaffected by anything that happens to the command's output
;  buffer.
; 
;  For asynchronous commands, the state of the command is stored in
;  the buffer-specific variable &excmd-cmd-active.  If the buffer
;  has a command associated with it, &excmd-cmd-active will be non-zero.
;  It will be 1 if the command is still executing, and -1 if it has
;  finished executing or was killed.  It may have been killed by
;  kill-command-execution or by a next-command-error issued while it was
;  still running.
;
;SEE ALSO
; 	grep.ml
;	compile.ml
;
;IMPLICIT INPUTS
; use-async-processes	If TRUE, runs commands as asynchronous processes,
;			otherwise uses filter-region.  Must be FALSE (0)
;			on systems without asynchronous process capability.
; 			(currently System III and System V).
;
;BUGS
;   Until asynchronous process exit hooks can be passed arguments, the
;   asynchronous command execution buffer will not have its mode
;   line updated until the first next-command-error is executed.
;
;HISTORY
; {2}	22-Feb-85  Eric S. Raymond (eric) at rabbit1
;	This works much better when  a global-rebind  is used  to map the
;	next-error function! Also updated the documentation.
;
; {1}	12-Jan-85  Mike Gallaher (mg) at unipress
;	Created.
;----------------------------------------------------------------------
(declare-global &excmd-last-buf)
(declare-buffer-specific &excmd-cmd-active)
(declare-buffer-specific &excmd-first-error)
(setq-default &excmd-cmd-active 0)		;  1 if active process
						;  0 if no command associated
						;    with this buffer
						; -1 if exited process
(setq-default &excmd-first-error 1)

(defun (execute-command		excmdcmd excmdbuf excmdfmt
	(setq excmdcmd	(arg 1 "Command: "))
	(setq excmdbuf	(arg 2 "Buffer: "))
	(setq excmdfmt	(arg 3 "Message format: "))

	(save-excursion
          (pop-to-buffer excmdbuf)
	  (setq &excmd-last-buf (current-buffer-name))
	  (setq &excmd-first-error 1)
          (setq needs-checkpointing 0)
	  (setq error-message-format excmdfmt)				; {2}
	  (setq mode-string "Executing")
 	  (setq mode-line-format (concat " Buffer: %b  %M (%m) " excmdcmd))
	  (erase-buffer)
	  (if use-async-processes
		(progn
	          (setq &excmd-cmd-active 1)
	          (if (>= (process-status excmdbuf) 0)
		     (kill-process excmdbuf))
		  ; the exit hook is called with buffer name as arg
	          (start-process excmdcmd excmdbuf "#excmd-exit-hook")	; {3}
	          (setq mode-string (concat
			  mode-string " (^X^K to kill)"))
		)
		(progn			; not async process
		  (beginning-of-file)
		  (set-mark)
		  (end-of-file)
		  (message "Executing " excmdcmd)
		  (sit-for 0)
		  (filter-region excmdcmd)
		  (#excmd-exit-hook)
		)
	  )
	)
	(novalue)
))

(defun (kill-command-execution		kcebuf
    (if use-async-processes
	(save-excursion
	    (setq kcebuf (current-buffer-name))
	    (if (! &excmd-cmd-active)
		(setq kcebuf &excmd-last-buf))
	    (pop-to-buffer kcebuf)
	    (kill-process kcebuf)
	    (#excmd-exit-hook kcebuf)
	))
))
	
(defun (#excmd-exit-hook
	(message "Command finished")
	(if (= &excmd-cmd-active 1)
	    (progn
		(setq mode-string "Done")
		(setq buffer-is-modified 0)
		(setq &excmd-cmd-active -1)
		(beginning-of-file)
		(set-mark)
		(end-of-file)))
        (novalue)						     
))

(defun (next-command-error	ncebuf
	(setq ncebuf (current-buffer-name))
	(if (! &excmd-cmd-active)
	    (setq ncebuf &excmd-last-buf))
	(pop-to-buffer ncebuf)
	(if use-async-processes
	    (progn
		(if (&  (= &excmd-cmd-active 1)
			(>= (process-status ncebuf) 0)
		   	(= (get-tty-string 
			"Command is still running, do you want to kill it? ")
		           "y"))
		    (kill-command-execution ncebuf))
	    ))
	(if &excmd-first-error
	    (progn
		(setq &excmd-first-error 0)
; Kludge!  This goes away when process exit hook is implemented.
;		(#excmd-exit-hook (current-buffer-name))
		(beginning-of-file)
		(set-mark)
		(end-of-file)
		(message "Parsing errors...")
		(parse-error-messages-in-region)
		(message "")
	    ))
        (next-error)
	))

(global-rebind "next-command-error"	"next-error"	"\^x\^n")
(bind-to-key "kill-command-execution"	"\^x\^k")
;(bind-to-key "execute-command" 	"\^x!")
(novalue)
; execute.ml ends here ------------------------------------------------------
