; help.ml
;
;               Copyright (c) 1985 - Unipress Software
;		Copyright (c) 1984 - Community Data Processing
;---------------------------------------------------------------------------
;
; SYNOPSIS
;	Provides help on Emacs commands, variables, keybindings, etc.
;
; KEYBINDINGS
; 	^_	help
;
; DESCRIPTION
;	This library makes several of the built-in Emacs help features
;	conveniently available through two-keystroke commands.  Typing the
;	help character ^_ invokes the help function, which reads the next
;	character and dispatches on it.  The available help subcommands are:
;
;	a	list all functions or variables APROPOS a keyword
;	b	display key BINDINGS, i.e. all the command characters
;	c	explain function performed by a COMMAND character(s)
;	d	use Info to DESCRIBE a function
;	e	EXECUTE some MLisp code
;	i	run INFO program to learn Emacs features
;	p	PRINT value of a variable 
;	s	SET a variable
;	t	edit Emacs TUTORIAL file to learn how to use Emacs
;	v	use Info to describe meaning of a VARIABLE
;	!	spawn subshell to run Unix commands
;
; SEE ALSO
;	describe-command
; 	describe-variable
; 	describe-bindings
; FILES
;	tutorial.txt       emacs/databases/info.???
; BUGS
;
; LATER
;   do w (whereis, ie binding for a function) with filter-region
;
; HISTORY
;  {3}  27-Dec-84  Mike Gallaher  (mg) at unipress
;	Made help subcommands dispatch from keymap, instead of using
; 	a giant if-then-else construct.
; 
;  {2}	27-Dec-84  David Medinets (dm) at unipress
;	Made c command use new get-tty-keys function
;
;  {1}  9-Jul-84 Scott Weikart (sw) at Community Data Processing
; 	Created.  All the common emacs documentation rolled into one help
;	command.
;
;----------------------------------------------------------------------

(autoload "learn" "learn.ml")

(setq-default &help-last-char 0)		;last doc char typed
(declare-buffer-specific &help-oldmap)
(define-keymap "&help-command-keymap")

(defun
  (help char
    (setq &help-oldmap (current-local-keymap))
    (use-local-map "&help-command-keymap")
    (message "help (? for help, q to quit): ")
  )

  (&help-menu depth			;show the user a menu
    (save-excursion
      (pop-to-buffer "Help")
      (erase-buffer)
      (if (= (setq depth (recursion-depth)) 0)
	 (insert-string "You are at top level.")
	 (insert-string "Recursive editing level " depth " (^C to abort).")
      )
; there are only ten lines here because that is normal for most terminals.
      (insert-string "\
  Type a key (q to quit):\n\
   a	list all functions or variables APROPOS a keyword\n\
   b	display key BINDINGS, i.e. all the command characters\n\
   c	explain function performed by a COMMAND character(s)\n\
   d	use Info to DESCRIBE a function\n\
   i	run INFO program to learn Emacs features\n")
      (insert-string "\
   t	edit Emacs TUTORIAL file to learn how to use Emacs\n\
   v	use Info to describe meaning of a VARIABLE\n\
 p ; s	PRINT value of a variable ; SET a variable\n\
 ! ; e	spawn subshell to run Unix commands ; EXECUTE some MLisp code\n\
")
      (setq needs-checkpointing 0)
      (setq buffer-is-modified 0)
      (beginning-of-file)
      (help)    
    )
  )

  (&help-q-command
    (use-local-map &help-oldmap)
  )

  (&help-space-command
    (execute-string &help-last-char)
  )

  (&help-a-command
    (apropos (arg 1 "Apropos keyword: "))
    (setq &help-last-char "a")
    (use-local-map &help-oldmap)
  )

  (&help-c-command  prompt
    (setq prompt "Type Command character(s): ")
    (describe-command (describe-key (get-tty-keys prompt)))
    (setq &help-last-char "c")
    (use-local-map &help-oldmap)
  )

  (&help-p-command  var
    (setq var 
          (get-tty-variable "Print value of variable named (use SPACE, ?): "))
    (message var " => " (execute-mlisp-line var))
    (setq &help-last-char "p")
    (use-local-map &help-oldmap)
  )

  (&help-s-command  var value
    (setq var (get-tty-variable "Variable to set (use SPACE, ?): "))
    (setq value (get-tty-string "Old value of " var " is "
				    (execute-mlisp-line var)
				    "; new value to use: "))
    (message var " <- " (set var value))
    (setq &help-last-char "s")
    (use-local-map &help-oldmap)
  )

  (&help-b-command
    (save-excursion (describe-bindings))
    (setq &help-last-char "b")
    (use-local-map &help-oldmap)
  )

  (&help-d-command
    (save-window-excursion (describe-command))
    (setq &help-last-char "b")
    (use-local-map &help-oldmap)
  )      

  (&help-e-command
    (execute-mlisp-line (get-tty-string "execute-mlisp-line "))
    (setq &help-last-char "e")
    (use-local-map &help-oldmap)
  )      

  (&help-i-command
    (save-window-excursion (info))
    (setq &help-last-char "i")
    (use-local-map &help-oldmap)
  )

  (&help-t-command
    (save-window-excursion (learn))
    (setq &help-last-char "t")
    (use-local-map &help-oldmap)
  )
    
  (&help-v-command
    (save-window-excursion (describe-variable))
    (setq &help-last-char "v")
    (use-local-map &help-oldmap)
  )
)

(setq &help-oldmap (current-local-keymap))
(error-occurred
  (use-local-map "&help-command-keymap")
  (progn i
    (setq i 0)
    (while (< i 128)
      (local-bind-to-key "&help-q-command" i)
      (setq i (+ i 1))
    )
  )
  (local-bind-to-key "&help-a-command"		"a"   "A")
  (local-bind-to-key "&help-b-command"		"b"   "B")
  (local-bind-to-key "&help-c-command"		"c"   "C")
  (local-bind-to-key "&help-d-command"		"d"   "D")
  (local-bind-to-key "&help-e-command"		"e"   "e")
  (local-bind-to-key "&help-i-command"		"i"   "I")
  (local-bind-to-key "&help-t-command"		"t"   "T")
  (local-bind-to-key "&help-p-command"		"p"   "P")
  (local-bind-to-key "&help-q-command"          "q"   "Q")
  (local-bind-to-key "&help-s-command"		"s"   "S")
  (local-bind-to-key "&help-v-command"		"v"   "V")
  (local-bind-to-key "&help-space-command"	" ")
  (local-bind-to-key "return-to-monitor"	"!")
  (local-bind-to-key "&help-menu"		"?")
)
(use-local-map &help-oldmap)

(bind-to-key "help" "\^_")
