;---------------------------------------------------------------------------
;NAME
;   incr-search.ml -- ITS-style incremental search and/or replace
;
;SYNOPSIS
;   incremental-search			-- incremental-search forward
;   incremental-re-search		-- ditto, with regular expressions
;   reverse-incremental-search		-- incremental-search backward
;   reverse-incremental-re-search	-- ditto, with regular expressions
;
;KEY BINDINGS
;   incremental-search		search-forward		^S
;   reverse-incremental-search	search-reverse		^R
;
;DESCRIPTION
;   Emacs  includes  a search  command  which  is  unusual  in  that  it  is
;"incremental";  it  begins  to  search  before you have finished typing the
;search string. As you type in the search string, Emacs shows you  where  it
;would  be  found.    When  you have typed enough characters to identify the
;place you want, you can stop. Typically  one binds incremental-search to ^S
;and reverse-incremental-search to ^R.
;
;   The command to search is ^S (incremental-search). ^S reads in characters
;and positions the cursor at the first occurrence of the characters that you
;have  typed.    If you type ^S and then F, the cursor moves right after the
;first "F".  Type an "O", and see the cursor move to after the  first  "FO".
;After  another  "O",  the  cursor  is after the first "FOO" after the place
;where you started the search.  At the same time, the "FOO"  has  echoed  at
;the bottom of the screen.
;
;   If you type a  mistaken  character,  you can rub it out.  After the FOO,
;typing a rubout makes the "O" disappear from  the  bottom  of  the  screen,
;leaving only "FO".  The cursor moves back to the "FO".  Rubbing out the "O"
;and "F" moves the cursor back to where you started the search.
;
;   When you are satisfied  with the place you have reached, you can type an
;ESC, which stops searching, leaving the cursor where the search brought it.
;Also, any command not specially meaningful in searches stops the  searching
;and  is then executed.  Thus, typing ^A would exit the search and then move
;to the beginning of the line.  ESC is necessary only if  the  next  command
;you  want  to  type  is a printing character, Rubout, ESC or another search
;command, since those are the characters that would not exit the search.
;
;   Sometimes you search for "FOO" and find it, but not the one you expected
;to find. There was a second FOO that you forgot about, before the  one  you
;were  looking  for.  Then type another ^S and the cursor will find the next
;FOO.  This can be done any number of times.  If you overshoot, you can  rub
;out  the  ^S's.    You  can also repeat the search after exiting it, if the
;first thing you type after entering another search (when  the  argument  is
;still empty) is a ^S.
;
;   If your string is not found at all, the minibuf says "Failing I-Search".
;The cursor is after the place where Emacs found as much of your  string  as
;it  could.    Thus, if you search for FOOT, and there is no FOOT, you might
;see the cursor after the FOO in FOOL.  At  this  point  there  are  several
;things you can do.  If your string was mistyped, you can rub some of it out
;and  correct it.  If you like the place you have found, you can type ESC or
;some other Emacs command to accept what the I-search offered.  Or  you  can
;type  ^G, which throws away the characters that could not be found (the "T"
;in "FOOT"), leaving those that were found (the "FOO" in "FOOT").  A  second
;^G at that point undoes the search entirely.
;
;   The ^G  "quit" command does  special  things during searches; just what,
;depends on the status of the search.  If the  search  has  found  what  you
;specified  and  is  waiting  for  input, ^G cancels the entire search.  The
;cursor moves back to where you started the search.  If ^G  is  typed  while
;the  search is actually searching for something or updating the display, or
;after search failed to find some of your input (having searched all the way
;to the end of the file), then only the characters which have not been found
;are discarded.  Having discarded them, the search  is  now  successful  and
;waiting for more input, so a second ^G will cancel the entire search.  Make
;sure  you  wait  for the first ^G to ding the bell before typing the second
;one; if typed too soon, the second ^G may be confused with  the  first  and
;effectively lost.
;
;   You can type ^R at any time to start searching  backwards.  If  a search
;fails because the place you started was too late in the file, you should do
;this.    Repeated  ^R's  keep looking for more occurrences backwards.  A ^S
;starts going forwards again.  ^R's can be rubbed  out  just  like  anything
;else.    If  you  know  that  you  want to search backwards, you can use ^R
;instead  of  ^S  to  start  the  search,  because  ^R  is  also  a  command
;(reverse-incremental-search) to search backward.  
; 
;   ^T toggles use of regular-expression metacharacters  in the  search. The
;entry points (incremental-re-search) and (reverse-incremental-re-search) act
;as described  above but start with regular expressions enabled.
;
;HISTORY
; 02-Aug-84  Eric S. Raymond at Rabbit Software
;	Added ^S and ^R global-rebinds.
; 04-Jul-84  Eric S. Raymond at Rabbit Software
;	Added documentation.
; 14-Jan-83  Jerry Agin (agin) at Carnegie-Mellon University
;	Added regular expression searches.  Can be invoked by calling
;	incremental-re-search or reverse-incremental-re-search, or by
; 	typing ^T within a search string.
; 30-Apr-82  Jerry Agin at CMU-750R
; 	- fixed case fold lossage
; 	- added search-exit-char
; 06-Nov-81  Jeffrey Mogul @ Stanford
;	- original version tried to reverse by re-defuning things.  This
;	  tended to fail quite often.  The direction of search is now
;	  tested explicitly.
;	- accepts <tab> as a "normal-character"
; A. Witkin 1/81
; 	Created.
;---------------------------------------------------------------------------

(declare-global I-search-string)
(setq I-search-string "")
(declare-global I-search-re)
(setq I-search-re 0)
(declare-global search-exit-char)
(if (= search-exit-char 0) (setq-default search-exit-char '\033'))

(defun

(&inc-initial
    (if (& (= (length string) 0)(> (length I-search-string) 0))
	(progn
	      (setq string I-search-string)
	      (setq is-re I-search-re))))

(&inc-message
    (message
	    (if failing "Failing " "")
	    (if is-forward "" "Reverse ")
	    (if is-re "R-E " "")
	    "I-search: "
	    string))

(&inc-forward-search
   (setq failing
	 (& (!= string "")
	    (error-occurred (if is-re
			       (re-search-forward string)
			       (search-forward string)))))
   (if (! failing)
       (progn (if (= string "")
		  (set-mark)
		  (region-around-match 0)
	      )
	      (setq goodlength (length string))))
   (&inc-message))

(&inc-reverse-search
   (setq failing
	 (& (!= string "")
	    (error-occurred (if is-re
			       (re-search-reverse string)
			       (search-reverse string)))))
   (if (! failing) 
       (progn
	     (set-mark)
	     (setq goodlength (length string))))
   (&inc-message))

(&inc-normal-char
    (setq string (concat string next))
    (if (& failing (! is-re))
	(&inc-message)
        (&inc-check)))

(&inc-check
    (if is-forward
	(&inc-forward-check)
	(&inc-reverse-check)))

(&inc-forward-check
      (exchange-dot-and-mark)
      (&inc-forward-search)
      (if failing (exchange-dot-and-mark)))

(s= s1 s2
    (setq s1 (arg 1))
    (setq s2 (arg 2))
    (if (= s1 s2)
	1
	(!= (length s1) (length s2))
	0
	(! case-fold-search)
	0
        (progn
	     (while (& (length s1) 
		       (c= (string-to-char s1) (string-to-char s2)))
		    (setq s1 (substr s1 2 -1))
		    (setq s2 (substr s2 2 -1)))
	     (! (length s1) ))))

(&inc-looking-at ans
    (if (= string "")
	1
	is-re
	(& (! (error-occurred (setq ans (looking-at string))))
	   ans)
	(save-excursion
	    (set-mark)
	    (goto-character (+ (dot) (length string)))
	    (s= (region-to-string) string))))

(&inc-reverse-check
   (if (&inc-looking-at string)
       (progn (setq failing 0)
	      (setq goodlength (length string))
	      (&inc-message))
       (&inc-reverse-search)))

(&inc-CG
   (if (= nextc '^G') 
      (if failing
	 (progn
	    (setq string (substr string 1 goodlength))
	    (setq failing 0)
	    (&inc-message))
	 (progn (setq ok 0)(goto-character start)))))

(&inc-CS 
   (if (= nextc '^S')
      (progn
	 (&inc-initial)
	 (setq is-forward 1)
	 (&inc-forward-search))))

(&inc-CR
   (if (= nextc '^R')
      (progn
	 (&inc-initial)
	 (setq is-forward 0)
	 (&inc-reverse-search)))) 

(&inc-DEL
    (if (= nextc '\177')
	(progn
	      (setq string (substr string 1 (- (length string) 1)))
	      (if (& failing (! is-re) (> (length string) goodlength))
			; There's a minor bug here.  (Can you find it?)
			; It's so obscure it's not worth fixing.
		  (&inc-message)
		  (&inc-check)))))

(&inc-CQ (if (= nextc '^Q')
	       (progn
		  (setq next
		     (char-to-string (setq nextc (get-tty-character))))
		  (if (&inc-meta-char) (setq next (concat "" next)))
		  (&inc-normal-char)
		  (setq nextc '^Q'))))

(&inc-ALT (if (= nextc search-exit-char) (setq ok 0)))

(&inc-CT			; use ^T to toggle regular-expression search
    (if (= nextc '^T')
	(progn
	      (setq is-re (! is-re))
	      (&inc-check))))

(&inc-special-char
      (|
	 (= nextc '^G')(= nextc search-exit-char)
	 (= nextc '^S')(= nextc '^R')
	 (= nextc '^Q')(= nextc '\177')
	 (= nextc '^T')))

(&inc-funny-char
      (| (& (!= nextc '^I') (< nextc ' ')) (> nextc '~')))

(&inc-meta-char
   (> nextc 127))

(incremental-search
    (&incremental-search 1 0))

(reverse-incremental-search
    (&incremental-search 0 0))

(incremental-re-search
    (&incremental-search 1 1))

(reverse-incremental-re-search
    (&incremental-search 0 1))

(&incremental-search
   string ok start failing is-forward goodlength push-back savemark is-re
   (setq is-forward (arg 1))
   (setq is-re (arg 2))
   (if (error-occurred (setq savemark (mark))) (setq savemark 0))
   (set-mark) (setq start (dot))
   (setq push-back -1)
   (setq string "")
   (setq ok 1) (setq failing 0) (setq goodlength 0)
   (&inc-message)
   (while ok (&inc-process-char))
   (setq I-search-string string)
   (setq I-search-re is-re)
   (if (>= push-back 0) (push-back-character push-back))
   (set-mark) (goto-character savemark) (exchange-dot-and-mark)
)

(&inc-process-char next nextc
   (setq next (char-to-string (setq nextc (get-tty-character))))
   (if 
      (&inc-funny-char)
      (if
	 (&inc-special-char)
	 (progn (&inc-CS)(&inc-CR)(&inc-CG)(&inc-DEL)(&inc-CQ)(&inc-ALT)
		(&inc-CT))
	 (progn
	    (setq push-back nextc)
	    (setq ok 0)))
      (&inc-normal-char))
   (if (!= nextc '^T') (setq I-search-string ""))))

(global-rebind "incremental-search" "search-forward" "\^S")
(global-rebind "reverse-incremental-search" "search-reverse" "\^R")
(novalue)
; incr-search ends here -----------------------------------------------------
