;---------------------------------------------------------------------------
; justify.ml
; 
;SYNOPSIS
; 
; (fill-paragraph)		- adjust text
; 
; (justify-line)		- make the line reach the right margin
; 
; (auto-justify-line)		- justify line when right margin is reached
; 
; (auto-justify-mode)		- turn on auto-justify-line 
; 
; (justify-paragraph)		- justify all the line in a paragraph
;
;KEYBINDINGS
; 
; justify-paragraph 	indent-C-procedure	ESC-j
;
;DESCRIPTION
;
;    To use auto-justify-mode simply load text-mode.ml.
;
;right-margin variable: defines the column of the right margin,
;	no character will appear after this column.
; 
;left-margin variable:  defines the column left margin, all lines
;	start in this column.
; 
;paragraph-indent variable: Paragraphs will be indented this many
;	characters from the left margin.
; 
;fill-paragraph function: this function aligns the left margin and
;	indents the paragraph.  It also places as many words as will fit 
;	on a line up to the right margin.  Fill-paragraph is also smart
;	when it comes to punctuation (i.e. there will be 2 spaces
;	after a period and one after a comma etc.)
; 
;justify-line function: this function inserts spaces in smart places
;	so line the last character of the line is at the right margin.
;	the right margin is defined by right-margin.
; 
;auto-justify-line function: this is the handler for the auto-justify-mode
;	it basically makes sure that everything is in the right place and
;	calls justify-line.  This function should never be used by anybody.
; 
;auto-justify-mode function: this sets up justify-line to occur
;	automatically.  That is when the user types past the right
;	margin the line will be justified.
; 
;justify-paragraph function: this function first calls fill-paragraph
;	and then it justifies all the lines in a paragraph.  It treats
;	the last line smartly (that is it doesn't justify the last line).
; 
;SEE ALSO
; 
; text-mode.ml
;
;HISTORY
; {2}	 4-Dec-84  David Medinets (dm) at unipress
;	Moved the (beginning-of-paragraph) and  (end-of-paragraph) to the
;	file paragraphs.ml.  Also added a variable called &just-space&tab,
;	this was to make the code prettier.
;
; {1}	29-Sep-84  David Medinets (dm) at unipress
;	Added code so that  justify-line will  not justify  any line that
;	needs more than 10 spaces to fill it out.
;
; Aug 17 1984 mcl (John S. Mc Laughlin at Unipress Software Inc.)
;     Created.
;
;----------------------------------------------------------------------

(declare-global &from-justify
                &just-space&tab
)
(setq &from-justify 0)

(declare-buffer-specific paragraph-indent)
(if (! (is-bound paragraph-indent))
    (setq-default paragraph-indent 5))

(if (> right-margin 200) (setq right-margin 69))

(if (! (is-bound auto-justify-p))
  (progn 
    (declare-global auto-justify-p)
    (setq auto-justify-p 0)
  )
)

(autoload "end-of-paragraph"	"paragraphs.ml")


(defun 
  (fill-paragraph
      (end-of-paragraph)
      (set-mark)
      (beginning-of-paragraph)
      (narrow-region)
      (delete-white-space)
      (error-occurred (re-replace-string "\n" " "))
      (error-occurred (re-replace-string "\\([	 ][	 ]*\\)" " "))
      (error-occurred 
        (re-replace-string "\\([.!?:][)}>""]*[.!?:]*[ ]\\)" "& "))
      (beginning-of-line)
      (insert-string prefix-string)
      (to-col (+ left-margin paragraph-indent))
      (while (! (eobp))
	(goto-character (+ (dot) (- (+ right-margin 1) (current-column))))
	(if (! (eobp))
	  (progn 
	    (if (! (looking-at "\\([ 	]\\)"))
	      (re-search-reverse "\\([ 	]\\)")
	    )
	    (delete-white-space)
	    (newline)
	    (insert-string prefix-string)
	    (to-col left-margin)
	  )
	)
      )
      (beginning-of-line)
      (if (looking-at ".") 
	(progn 
	  (end-of-line)
	  (newline)
	)
      )
      (if (! &from-justify) (widen-region))
  )
)

      
(defun 
  (justify-paragraph    
      (setq &from-justify 1)
      (fill-paragraph)
      (beginning-of-file)
      (while (! (eobp))
	(previous-line)
	(justify-line)
	(next-line)
	(next-line)
	(end-of-line)
      )
      (setq &from-justify 0)
      (widen-region)
  )
)
 
(defun 
  (justify-line #words #spaces #sub line#
    (setq line# (line-number))
    (end-of-line)
    (delete-white-space)
    (if (> (current-column) (- right-margin 10))         ; {1}
       (progn						 ; {1} 
          (beginning-of-line)				 
	  (setq #words 0)
          (re-search-forward "\\([ 	]*\\)")
	  (while (& (! (eolp)) (! (eobp)) (= line# (line-number)))
	     (if (error-occurred (re-search-forward  "\\([ 	]\\)"))
		(end-of-line)
	     )
	     (++ #words)
	  )
          (if (!= line# (line-number))
             (previous-line)
	  )
          (if (& #words (! (eobp)))
             (progn
	        (-- #words)
                (end-of-line)
	        (setq #spaces (+ 1 (- right-margin (current-column))))
	        (beginning-of-line)
                (re-search-forward "\\([ 	]*\\)")
	        (while (& (> #spaces 0) (> #words 0))
                   (re-search-forward "\\([ 	]\\)")
	           (if (setq #sub (/ #spaces #words))
	              (provide-prefix-argument #sub (insert-character ' '))
		   )
	           (if (/ #words 2)
	              (if (/ (% #spaces #words) (/ #words 2))
	                 (progn 
		            (++ #sub)
		            (insert-character ' ')
			 )
		      )
		   )
	           (-= #spaces #sub)
	           (-- #words)
		)
	     )
	  )
       )                                                  ; {1} 
    )                                                     ; {1} 
  )							  
)

(defun 
  (auto-justify-line
    (re-search-reverse "\\([\t \n]\\)")
    (if (looking-at ".")
      (progn
	(forward-character)
	(delete-white-space)
	(newline)
	(end-of-line)
	(save-excursion  
	  (previous-line)
	  (justify-line)
	)
      )
      (progn
	(forward-character)
	(end-of-line)
      )
    )
    (last-key-struck)
  )
)

(defun  
  (auto-justify-mode 
    (setq auto-justify-p 1)
    (set-auto-fill-hook  "auto-justify-line")
  )

  (check-auto-justify
    (if auto-justify-p (auto-justify-mode))
  )
)
