;-----------------------------------------------------------------------------
;NAME
;   killring.ml -- kill and unkill text a la TOPS-20
;
;SYNOPSIS
;   See below
;
;KEY BINDINGS
;    kill-region	delete-to-killbuffer		^W
;    copy-region					ESC-w, ESC-W
;    kill-lines		kill-to-end-of-line		^K
;    copy-lines						ESC-k, ESC-K
;    kill-word		delete-next-word		ESC-d, ESC-D
;    backward-kill-word	delete-previous-word		ESC-h, ESC-del
;    kill-char		delete-next-character		^D
;    backward-kill-char	delete-previous-character	del (\0177), ^h
;    append next kill					ESC-^W
;    unkill		yank-from-killbuffer		^Y
;    unkill-pop						ESC-Y
;
;DESCRIPTION
;   This package uses the rebind feature to replace the default emacs text-
;killing commands, which delete to one special buffer called the kill-buffer.
;
;   These commands to delete words, lines, and regions actually send text to
;a ring of killbuffers, where they can be yanked back.  Multiple kill commands
;in succession will concatenate text to the same buffer, so a single unkill
;can bring it all back. The append-next-kill command can be used to continue
;appending to the last-used killbuffer after non-delete commands (rather than
;moving to a new one as would normally occur).
;
;   The unkill-pop command can cycle the kill ring to retrieve previously-
;killed stuff.
;
;   There are usually four buffers in the killring.  If you want more buffers
;in the ring (say 8), execute the following mlisp assignment BEFORE you load
;this file:
;
; 	(setq-default nrings 8)
;
;or else execute the following AFTER you load this file:
;
;	(set-nrings 8)
;
;   The kill-lines function will behave pretty much the same as the old
;delete-to-end-of-line did, unless you want something better.  The improved
;version bases its behavior on the horizontal position of the cursor at the
;time the command is issued.  If the cursor is at the beginning of the line,
;the command will assume you want to kill the entire line, including the
;return at the end.  If you're at the end of the line, then it will remove
;the return separating this line from the next.  Otherwise, it will kill
;just to the end of the line.  To get this all to happen, execute the 
;following mlisp assignment
;
;	(setq-default &kill-lines-magic 1)
;
;   The variable &MaxKillSize is initialized to a value of zero.  If you set
;it to a value such as 400 with the call
;
; 	(setq-default &MaxKillSize 400)
;
;then any kill function will ask the user for confirmation if the region
;being killed exceeds the size specified.
;
;   Terminal-customization packages may want to define a killring-hook to
;execute at the end of this package's load in case the key assignments that
;aren't rebinds (copy-region, copy-lines, append-next-kill and unkill-pop)
;clobber things.
;
;HISTORY:
; 11-Aug-84  John S. Mc Laughlin (mcl) at UniPress Software Inc.
;	Commented out the bindings of kill-char and backward-kill-char.
;	I feel that they cause the killring to cycled through to fast.
; 27-Jun-84  Eric Raymond (eric) at Rabbit Software
;	Added rebinds and killring-hook, changed some function names,
;	added documentation.
; 08-Dec-82  Jerry Agin (agin) at Carnegie-Mellon University
; 	Made some of Duane's code more efficient.
;	Killing no longer moves the mark (but copying lines and
;	unkilling still do).
; 03-Dec-82  Duane Williams (dtw) at Carnegie-Mellon University
;	Fixed bug in forward-lines so that kill-lines works properly on
;	empty lines when &kill-lines-magic is 1 and user tracks eol on 
; 	^N-^P.  Also added kill-character and backward-kill-character,
; 	and made kill-region warn the user when killing a large region.
; 28-Jul-82  Jerry Agin (agin) at Carnegie-Mellon University
;	Moved the binding of append-next-kill from esc-a to
;	esc-^W to avoid a conflict with backward-sentence.
; 14-Jul-82  Jerry Agin (agin) at Carnegie-Mellon University
;	Created.
;-----------------------------------------------------------------------------

(defun
    (region-size val
	(setq val (- (dot) (mark)))
	(if (>= val 0)
	    val
	    (- 0 val)
	)
    )
)

(defun
    (kill-region
	(if (| (<= &MaxKillSize 0)
	       (<= (region-size) &MaxKillSize)
	       (= (substr (get-tty-string 
			       (concat "Do you really want to kill " 
					(region-size) " characters? [no] "))
			   1 1)
		  "y")
	    )
	    (progn
		  (if (= (previous-command) -11)
		      (progn
			  (c-append-region)
			  (erase-region))
		      (progn
			  (next-killbuffer)
			  (delete-region-to-buffer killbuffer)))
		  (setq this-command -11)
	    )
	    (setq this-command (previous-command))
	)
    )
)

(defun
    (copy-region
	(if (= (previous-command) -11)
	    (c-append-region)
	    (progn
		(next-killbuffer)
		(copy-region-to-buffer killbuffer)))
	(setq this-command -11))
)

(defun
    (c-append-region
	(if (>= (dot) (mark))
	    (append-region-to-buffer killbuffer)
	    (progn
		(exchange-dot-and-mark)
		(yank-buffer killbuffer)
		(copy-region-to-buffer killbuffer))))
)

(defun
    (append-next-kill
	(setq this-command -11))
)

(defun
    (unkill				; leaves region around restored text
	(set-mark)
	(yank-buffer killbuffer)
	(setq this-command -25)
    )
)

(defun
    (unkill-pop
	(if (= (previous-command) -25)
	    (progn
		  (delete-region-to-buffer killbuffer)
		  (previous-killbuffer)
	    )
	)
	(unkill)
    )
)

(defun
    (set-up-killbuffer
	(setq killbuffer (concat "KB-" ringpos)))
)

(defun
    (next-killbuffer
	(setq ringpos (+ ringpos 1))
	(if (> ringpos nrings)
	    (setq ringpos (- ringpos nrings)))
	(set-up-killbuffer))
)

(defun
    (previous-killbuffer
	(setq ringpos (- ringpos 1))
	(if (< ringpos 1)
	    (setq ringpos (+ ringpos nrings)))
	(set-up-killbuffer))
)

(defun
    (forward-lines
	(if (& &kill-lines-magic (bolp))
	    (if (error-occurred		; beginning of line--kill whole lines
		    (prefix-argument-loop (search-forward "\n"))
		)
		(end-of-file)
	    )
	    (error-occurred
		(prefix-argument-loop
		    (if (eolp)
			(forward-character)	; end of line--kill crlf
			(end-of-line))))))	; else--kill to end of line
)

(defun
    (kill-lines		; to append to killbuffer, set dot before killing
	(save-excursion
	    (set-mark)
	    (provide-prefix-argument prefix-argument (forward-lines))
	    (kill-region)))
)

(defun
    (copy-lines
	(set-mark)
	(provide-prefix-argument prefix-argument (forward-lines))
	(copy-region))
)

(defun
    (kill-word
	(save-excursion
	    (set-mark)
	    (provide-prefix-argument prefix-argument (forward-word))
	    (kill-region)))
)

(defun
    (backward-kill-word
	(save-excursion
	    (set-mark)
	    (provide-prefix-argument prefix-argument (backward-word))
	    (kill-region)))
)

(defun
    (kill-char
	(save-excursion
	    (set-mark)
	    (provide-prefix-argument prefix-argument (forward-character))
	    (kill-region)))
)

(defun
    (backward-kill-char
	(save-excursion
	    (set-mark)
	    (provide-prefix-argument prefix-argument (backward-character))
	    (kill-region))
    )
)

(defun
    (set-nrings
	(save-excursion
	    (setq nrings (arg 1 "Number of kill rings: "))
	    (setq ringpos (+ nrings 1))
	    (while (> ringpos 1)
		   (setq ringpos (- ringpos 1))
		   (set-up-killbuffer)
		   (temp-use-buffer killbuffer)
		   (setq needs-checkpointing 0)
	    )
	)
    )
)

;-----------------------------------------------------------------------------
;Here's the package initialization code
;-----------------------------------------------------------------------------

(declare-global nrings)
(if (= nrings 0) (setq-default nrings 4))
(declare-global ringpos)
(declare-global killbuffer)
(if (! (is-bound &kill-lines-magic))
    (setq-default &kill-lines-magic 0))
(if (! (is-bound &MaxKillSize))
    (setq-default &MaxKillSize 0))

(set-nrings nrings)

(global-rebind "kill-region" "delete-to-killbuffer" "\^W")
(bind-to-key "copy-region" "\ew")
(bind-to-key "copy-region" "\eW")
(global-rebind "kill-lines" "kill-to-end-of-line" "\^K")
(bind-to-key "copy-lines" "\ek")
(bind-to-key "copy-lines" "\eK")
(if (! (global-rebind "kill-word" "delete-next-word" "\ed"))
	(bind-to-key "kill-word" "\eD"))
(if (! (global-rebind "backward-kill-word" "delete-previous-word" "\e\0177"))
	(bind-to-key "backward-kill-word" "\e\h"))
;(if (! (global-rebind "backward-kill-char" "delete-previous-character" "\0177"))
;	(bind-to-key "backward-kill-char" "\^h"))
;(global-rebind "kill-char" "delete-next-character" "\^D")
(bind-to-key "append-next-kill" "\e\^W")
(global-rebind "unkill" "yank-from-killbuffer" "\^Y")
(if (! (global-rebind "unkill-pop" "yank-buffer" "\ey"))
	(bind-to-key "unkill-pop" "\eY"))
(error-occurred (killring-hook))

; killring.ml ends here ------------------------------------------------------
