;----------------------------------------------------------------------------
;NAME
;   paragraphs.ml -- functions to work with paragraph delimiters
;
;SYNOPSIS
;  forward-paragraph ------- jump to the start of the next paragraph
;  backward-paragraph ------ jump to the begining of the current paragraph
;  beginning-of-paragraph -- move cursor to previous paragraph delimiter
;  end-of-paragraph -------- move cursor to next paragraph delimiter
;
;KEY BINDINGS
;
;DESCRIPTION
;   This functions allow the user to step through a document in paragraph
;units. The global paragraph-delimiters may be set to define the RE search
;string used to find paragraph boundaries.
;
;beginning-of-paragraph function: this function moves the dot to the line
;	after the nearest blank line above the dot or to the beginning of
;	the buffer.
; 
;end-of-paragraph function: this function moves to the nearest blank line
;	below the dot or to the end of the buffer.
; 
;LATER
;  These functions would be faster if they didn't do so many re-searches.
; A lot of those could be replaced by region-around-matches, but that
; would need push-mark and pop-mark working.
;
;HISTORY
; {2}	 1-Feb-85  David Medinets (dm) at unipress
;	forward-paragraph and backward-paragraph took no notice of prefix
;	arguments.  This is fixed now.
;
; {1}	 4-Jul-84  Eric S. Raymond (eric) at Rabbit Software
;	Added this documentation header.
;----------------------------------------------------------------------------

(if (! (is-bound paragraph-delimiters))
    (progn
	(declare-global paragraph-delimiters)
	(setq paragraph-delimiters "^\n\\|^[@.'].*\n")
    ))

(defun
  (forward-paragraph
    (prefix-argument-loop 
      (beginning-of-line)
      (re-search-forward paragraph-delimiters)
      (while (looking-at "")
        (search-forward "")
      )
    )
  )
)

(defun
  (backward-paragraph
    (prefix-argument-loop 
      (beginning-of-line)
      (previous-line)
      (while (& (looking-at paragraph-delimiters) (! (bobp)))
        (progn
          (previous-line)
          (beginning-of-line)
        )
      )
      (if (error-occurred (re-search-reverse ""))
        (beginning-of-file)
      )
      (while (looking-at "")
	(search-forward "")
      )
    )
  )
)

(defun 
  (beginning-of-paragraph
    (if (error-occurred (re-search-reverse paragraph-delimiters))
      (beginning-of-file)
      (re-search-forward paragraph-delimiters)
    )
    (if (looking-at "\n") (next-line))
    (beginning-of-line)
  )
)

(defun 
  (end-of-paragraph
    (if (error-occurred (re-search-forward paragraph-delimiters))
      (end-of-file)
      (re-search-reverse "")
    )
  )
)

; paragraphs.ml -------------------------------------------------------------
