;-----------------------------------------------------------------------------
;NAME
;   process.ml -- support code for multiple processes under Emacs
;
;SYNOPSIS
;
;KEY BINDINGS
;   See below.
;
;DESCRIPTION
;   This package provides assistance for using the process control facilities
;of emacs to run interactive programs in windows.  It also provides a facility
;to allow the current instance of Emacs to serve as your editor for programs
;that need editors (hg, post, etc.).
;   Any keys bound to either (newline) or (newline-and-indent) will be rebound
;to point at the (pr-newline) function that sends the current line to the
;process tied to the current window.
;   ASCII delete and control-\ keys are locally bound to their usual interrupt
;functions. The former implies that some terminals may lose their backward
;character-delete functions.
;   Control-d has its usual end-of-stream meaning only when you're at the end
;of the buffer; otherwise it executes its current global binding.
;   Escape-return is bound to pr-newline-kill. This function is similar to
;pr-newline except that if it is invoked on a line in the middle of the buffer
;of the buffer, that line is set to the subprocess and the rest of the buffer
;buffer is erased.
;   The (grab-last-line) function (bound to \e=) inserts the last line that
;was sent to the process at dot. The (grab-current-line) function (bound to
;\e+) grabs the current line and puts it at the end of the buffer.
;   ^X^K is locally bound to a function that sends any remaining output from
;the process to the bit bucket.
;   ESC-j is bound to a function that shell-expands the filename dot is
;currently sitting on.
;
;   A full description of the facilities will not be given here as it is
;quite long. The user is referred to Section 18 of the unicorn book (Gosling's
;Emacs manual) for details.
;
;   The function (shell-hook) is available for user customization of this
;mode. It is executed each time a shell window is created, after the keymap
;initializations.
;
;HISTORY
 ; {1}	 1-Feb-85  David Medinets (dm) at unipress
 ;	In the function send-eot it checks to see if  you are  at the end
 ;	of the buffer.  it you are not it will find the global-binding of
 ;	whatever invoked it and execute  that.   The problem  was that it
 ;	ignored the prefix argument.  I have fixed this.
 ;
;   16-Jul-84  Eric S. Raymond at Rabbit Software
;	Made the keymapping work, added rebinds, documented how to use it.
;   04-Jul-84  Eric S. Raymond at Rabbit Software
;	Added this documentation header.
;-----------------------------------------------------------------------------

; marker for typeahead
(declare-buffer-specific process-output-marker)

; User option:
(setq-default &Edit-delete-window 1)	; zap edit window when leaving

; for grab-last-line function
(declare-buffer-specific last-line)

; For editing functions.
(declare-buffer-specific &Edit-return-buffer)

; Figure out which shell to use.
(if (! (is-bound &shell-command))
    (progn (setq-default &shell-command (getenv "SHELL"))
	   (if (= &shell-command "") (setq &shell-command "/bin/sh"))))

(if (| (! (is-bound shell-prompt-template))
       (= shell-prompt-template ""))
    (setq-default shell-prompt-template
	(if (= (substr &shell-command -3 3) "csh") "^% " "^$ ")))

(if (! (is-bound &editor-name))
    (setq-default &editor-name (concat emacs-helper-bin-dir "emacsedit")))

(if (! (is-bound &editor-set-command))
    (setq-default &editor-set-command
	(if (= (substr &shell-command -3 3) "csh")
	    (concat "setenv EDITOR " &editor-name ";exec ")
	    (concat "EDITOR=" &editor-name "; export EDITOR;"))))

; Bound to carriage return.  This function sends the last line of input to
; the current process.  If we are in the middle of the buffer, we guess
; about how much to send to the process, then we copy it to the end and send
; it.  If we're at the end of the buffer, we send the current region.
(defun
    (pr-newline
	(pr-do-newline 0)
    )
)

; Similar to pr-newline except that if it is invoked on a line in the middle
; of the buffer, that line is sent to the subprocess and the rest of the
; buffer is erased.
(defun
    (pr-newline-kill
	(pr-do-newline 1)
    )
)

; Shared code for pr-newline and pr-newline-kill
(defun
    (pr-do-newline
	(end-of-line)
	(if (eobp)
	    (progn
		  (newline)
		  (if (= (dot) process-output-marker)
		      (progn	; hack for special case
			    (backward-character)
			    (beginning-of-line)
			    (if (looking-at shell-prompt-template)
				(region-around-match 0))
		      )
		      (goto-character process-output-marker))
		  (set-mark)
		  (end-of-file)
		  (setq last-line (region-to-string))
	    )
	    (progn (beginning-of-line)
		   (if (looking-at shell-prompt-template)
		       (region-around-match 0))
		   (set-mark)
		   (end-of-line)
		   (forward-character)
		   (setq last-line (region-to-string))
		   (if (= (arg 1) 0)
		       (progn
			     (end-of-file)
			     (set-mark)
			     (insert-string last-line)
		       )
		       (progn
			     (set-mark)
			     (end-of-file)
			     (erase-region)
		       )
		   )
	    )
	)
	(string-to-process (active-process) last-line)
	(setq process-output-marker (dot))
    )
)

; Expand a file name inline
(defun (expand-inline-filename
	   (save-excursion
	       (set-mark)
	       (push-back-character ' ')
	       (while (& (! (bolp))
			 (| (>= (preceding-char) 'A')
			    (= (preceding-char) '$')
			    (& (>= (preceding-char) '.')
			       (<= (preceding-char) '9'))))
		      (backward-character))
	       (push-back-string (region-to-string))
	       (erase-region))
	   (insert-string (get-tty-file ": file ") " ")))


; Send an end-of-file signal to the process if we're at the end of the
; buffer, else just do what this key normally does.

; On some systems eot-process will not work. In that case use the
; (string-to-process) version of (send-eot).

(defun
  (send-eot
    (if (eobp)
       (progn
         (setq process-output-marker (dot))
	 (if eot-process-works
	    (eot-process (active-process))
	    (string-to-process (current-buffer-name) "\^d")
         )
       )
       (prefix-argument-loop 
         (execute-mlisp-line (concat "("
                                     (global-binding-of (last-key-struck))
   			             ")"
                             )
         )
       )
    )
  )
)

; Simulate keyboard interrupts.
(defun
    (send-int-signal
	(end-of-file)
	(setq process-output-marker (dot))
	(int-process (active-process))))

(defun
    (send-quit-signal
	(end-of-file)
	(setq process-output-marker (dot))
	(quit-process (active-process))))

; Insert the last line the user sent to the process in this buffer.
(defun
    (grab-last-line
	(end-of-file)
	(insert-string last-line)
	(delete-previous-character)
    )
)

; Insert this line at end of buffer for user.
(defun 
    (grab-current-line prompt line
	; Find current prompt.
	(save-excursion 
	    (end-of-file)
	    (set-mark)
	    (beginning-of-line)
	    (setq prompt (region-to-string)))
	; If this line has same prompt, skip it.
	(beginning-of-line)
	(if (looking-at (quote prompt))
	    (provide-prefix-argument 
		(length prompt)
		(forward-character)))
	; Now grab text and insert it at end.
	(set-mark)
	(end-of-line)
	(setq line (region-to-string))
	(end-of-file)
	(set-mark)
	(insert-string line)
	(novalue)
    )
)

; Clever hook to cd emacs with shell cds.
; problem: the mark is not being set correctly at times.
(defun
  (shell-cd
    (if (= (- (dot) (mark)) 2)
       (progn
         (if (error-occurred (cd (get-tty-file ": cd ")))
	    (progn
              (delete-previous-word)
	       (message "There is no such directory")
            )
	    (progn
	      (insert-string " " (working-directory))
	      (pr-newline)
            )
         )
	 0
       )
       1
    )
  )
)

(defun
    (lisp-kill-output
	(end-of-file)
	(beginning-of-line)
	(set-mark)
	(previous-line)
	(re-search-reverse "^[0-9][0-9]*\.")
	(next-line)
	(erase-region)
	(backward-character)
	(insert-string "    [output flushed]")
	(end-of-file)
	(setq process-output-marker (dot))
    )
)

(defun 

; trash remaining output from the current process
    (generic-kill-output prompt
	(end-of-file)
	(save-excursion 
	    (set-mark)
	    (beginning-of-line)
	    (setq prompt (region-to-string)))
	(beginning-of-line)
	(set-mark)
	(re-search-reverse (concat "^" (quote prompt)))
	(if (! (dot-is-visible)) (line-to-top-of-window))
	(next-line)
	(erase-region)
	(insert-string "[output flushed]\n")
	(end-of-file)
	(setq process-output-marker (dot))
    )
)

; The editor hacks.
(defun 
    (Process-edit-message old-buffer edit-buffer file
	(setq old-buffer (current-buffer-name))
	(setq edit-buffer (concat old-buffer "-edit"))
	(pop-to-buffer edit-buffer)
	(setq &Edit-return-buffer old-buffer)
	(setq file (arg 1 "File: "))
	(if (= (file-exists file) 0)
	    (progn
		  (erase-buffer)
		  (write-named-file file)
		  (read-file file))	; force auto-executes to take effect
	    (read-file file))
	(local-rebind "Process-done-edit" "exit-emacs" "\^C")
	(local-rebind "Process-done-edit" "write-current-file" "\^X\^S")
	(local-rebind "Process-done-edit" "write-file-exit" "\^X\^F")
	(message "Type ^C to return to your program.")
	(sit-for 0)
    )
)

(defun 
    (Process-done-edit buf
	(write-current-file)
	(setq buf &Edit-return-buffer)
	(if &Edit-delete-window
	    (delete-window))
	(pop-to-buffer buf)
	(end-of-file)
	(pr-newline)
    )
)

; Go to a shell buffer and run a shell there.
(defun
    (new-shell name
	(setq name (arg 1 ": new-shell (buffer name) "))
	(pop-to-buffer name)
	(if (< (process-status name) 0)
	    (progn
		  (use-abbrev-table "shell")
		  (use-syntax-table "shell")
		  (setq abbrev-mode 1)
		  (setq needs-checkpointing 0)
		  (setq wrap-long-lines 0)
		  (local-bind-to-key "generic-kill-output" "\^X\^K")
		  (local-bind-to-key "expand-inline-filename" "\e\^J")
		  (error-occurred
			(local-rebind "pr-newline" "newline"))
		  (error-occurred
			(local-rebind "pr-newline" "newline-and-indent"))
		  (local-bind-to-key "pr-newline-kill" "\e\^M")
		  (local-bind-to-key "send-eot" "\^d")
		  (local-bind-to-key "send-int-signal" "\177")
		  (local-bind-to-key "send-quit-signal" "\^\")
		  (local-bind-to-key "grab-last-line" "\e=")
		  (local-bind-to-key "grab-current-line" "\e+")
		  (if (= shell-prompt-template "") ; hack for setq-default bug
		     (setq shell-prompt-template
			   (if (= (substr &shell-command -3 3) "csh")
				"^% "
				"^$ ")
		      )
		  )
		  (error-occurred (shell-hook))
		  (end-of-file)
		  (setq process-output-marker (dot))
		  (set-mark)
		  (start-filtered-process
		      (concat &editor-set-command &shell-command)
		      name
		      "process-filter")
		  ))
	(end-of-file)
	(novalue)
    )
)

(defun
    (shell
	  (new-shell "shell")
    )
)

; Like the shell command for lisp.
(defun
    (lisp
	 (pop-to-buffer "lisp")
	 (setq needs-checkpointing 0)
	 (if (bobp)
	     (progn
		   (electric-lisp-mode)
		   (local-bind-to-key "lisp-kill-output" "\^X\^K")))
	 (if (< (process-status "lisp") 0)
	     (progn
		  (error-occurred
			(local-rebind "pr-newline" "newline"))
		  (error-occurred
			(local-rebind "pr-newline" "newline-and-indent"))
		   (local-bind-to-key "pr-newline-kill" "\e\^M")
		   (local-bind-to-key "send-eot" "\^d")
		   (local-bind-to-key "send-int-signal" "\177")
		   (local-bind-to-key "send-quit-signal" "\^\")
		   (local-bind-to-key "grab-last-line" "\e=")
		   (local-bind-to-key "grab-current-line" "\e+")
		   (error-occurred (shell-hook))
		   (end-of-file)
		   (setq process-output-marker (dot))
		   (set-mark)		   
		   (start-filtered-process
		       (concat &editor-set-command "cmulisp")
		       "lisp"
		       "process-filter")
	      )
	 )
	 (end-of-file)
	 (novalue)
    )
)

; insert output at process-output-marker
(defun
    (process-filter pr-dot pr-buff
	(setq pr-buff (dot))
	(temp-use-buffer MPX-process)
	(setq pr-dot (dot))
	(goto-character process-output-marker)
	(insert-string (process-output))
	(if (= pr-dot process-output-marker)
	    (setq pr-dot (dot))		     ; move dot after new text
	)				     ; if was at output-marker
	(if (= (mark) process-output-marker)
	    (set-mark)			     ; drag mark along too
	)
	(setq process-output-marker (dot))   ; move our mark after new text
	(goto-character pr-dot)		     ; where users dot was
	(pop-to-buffer pr-buff)
	(sit-for 0)
    )
)

; Filter for share file input.
(defun 
    (Share
	  (execute-mlisp-line (process-output))
    )
)

(insert-share-filter "Share")

; Set up global stuff.
(save-excursion
    (temp-use-buffer "shell")
    (use-abbrev-table "shell")
    (setq abbrev-mode 1)
    (define-local-abbrev "~" (getenv "HOME"))
    (define-hooked-local-abbrev "cd" "cd" "shell-cd")
    (use-syntax-table "shell")
    (modify-syntax-entry "w    ~")
    (setq right-margin 10000)
    (novalue)
)

; process.ml ends here -------------------------------------------------------
