;-----------------------------------------------------------------------------
;NAME
;   spell.ml -- correct spelling errors
;
;SYNOPSIS
;   spell			invoke the spell(1) facility on the current
;				buffer
;   correct-spelling-mistakes	interactively steps through possible
;				misspelled words, asking if it should be
;				left alone or corrected.
; 
;KEY BINDINGS
;   None.
;
;DESCRIPTION
;   Calling spell on the current buffer opens an 'Error Log' buffer where
;words that may be misspellings are posted as spell(1) runs. When the run is
;completed, the spell code will take you through a series of queries through
;which all the detected spelling mistakes in the buffer can be corrected.
;
;   Options can be passed to spell by setting the string variable spell-flags;
;in particular a (setq-default spell-flags "-b") will cause it to check for
;British spelling.
;
;HISTORY
;   04-July-84  Eric S. Raymond at Rabbit Software
;	Added this documentation and the spell-flags feature.
;-----------------------------------------------------------------------------

(if (! (is-bound spell-flags))
   (setq-default spell-flags "")
)

(if (! (is-bound &spell-buffer))
   (setq-default &spell-buffer "spell output")
)

(if (! (is-bound spell-production))
   (setq-default spell-production (concat "spell " spell-flags))
)

(autoload "execute-command" "execute.ml")

(defun
  (correct-spelling-mistakes word action continue
    (setq continue 1)
    (while (= continue 1)
      (save-excursion
        (temp-use-buffer &spell-buffer)
	(beginning-of-file)
	(set-mark)
	(end-of-line)
	(setq word (region-to-string))
	(if (error-occurred (forward-character))
	   (error-message "That was the last of the misspellings.")
        )
	(delete-to-killbuffer)
      )
      (beginning-of-file)
      (error-occurred (search-forward word))
      (message word " ? (r=replace) (^g or e =exit) (n=no action) ")
      (setq action (get-tty-character))
      (beginning-of-line)
      (if (| (= action '^G') (= action 'e'))
         (progn 
           (delete-buffer &spell-buffer)
           (setq continue 0)
         )
          (= action 'r')
         (progn newd
           (setq newd (get-tty-string (concat word " => ")))
	   (beginning-of-file)
	   (message "performing the query-replace")
	   (sit-for 20)
           (error-occurred (query-replace-string word newd))
         )
      )
    )
    (novalue)
  )
)

(defun
  (spell
    (save-window-excursion 
      (mark-whole-buffer)
      (copy-region-to-kill-buffer)
      (temp-use-buffer &spell-buffer)
      (yank-from-killbuffer)
      (mark-whole-buffer)
      (message "Filtering for possible errors. Please wait.") (sit-for 20)
      (filter-region spell-production)
    )
    (correct-spelling-mistakes)
    (novalue)
  )
)

(defun 
  (new-spell
    (setq spell-production (concat spell-production " " (current-file-name)))
    (execute-command spell-production &spell-buffer "")
    (novalue)
  )
)
; spell.ml -------------------------------------------------------------------
