;---------------------------------------------------------------------------
;NAME	tabstops.ml -- set tab stop columns
;
;SYNOPSIS
;   edit-tab-stops	-- modify the current tab stop settings
;   tab-to-tab-stop	-- insert whitespace to move to the next tab column
;
;KEYBINDINGS
;   tab-to-tab-stop				TAB
;
;DESCRIPTION
; This package maintains a set of "tab stops", which behave much the same
; way as those on a typewriter.  Each time you hit the TAB key, enough
; whitespace is inserted at the current cursor position to make the cursor
; end up at the next tab stop column.  To use this package, load tabstops.ml
; and locally bind the function tab-to-tab-stop to the TAB key in each buffer
; in which you want to use tab stops.  (This binding is usually done in
; mode packages;  for example, you might write a letter.ml that loads
; tabstops.ml and does the local-bind-to-key, sets the margins, etc.)
;
; Tab stops are maintained in a buffer-specific variable tab-stops, so each
; buffer has its own set of tab stops.  The tab stop settings are stored in
; a string, called a tab-stop template, each of whose characters represent
; a column on the display.  A 'T' in a character position indicates a
; tab-stop in the corresponding column.  For example, the tab-stop template
; "       T       T       T       T       T       T       T"
; defines a tab stop every eight columns.
; 
; To set the default tab stops for all buffers, use set-default or
; setq-default to assign a tab-stop template string to the variable tab-stops.
; These tab settings will be for all new buffers.  If you want different
; tab settings for a particular buffer, switch to that buffer and use
; setq to assign the tab-stop template to the variable tab-stops.
; Alternatively, you can use the edit-tab-stops function to edit the tab-stop
; template in a scratch buffer.  If tab-stops is set to a null string, then
; tab-to-tab-stop will insert a tab character in the buffer.
;
;SEE ALSO
;  to-col			function to insert whitespace to move to a
;				specified column
;  use-spaces-in-to-col		variable.  When set, to-col will insert only
;				spaces, never tabs.  If not set, to-col will
;				use whatever combination of spaces and tabs
;				will require the fewest characters.
;  tab-size			Buffer-specific variable.  Determines to how
; 				many spaces on the display a tab will expand.
; 				Affects only the display, does not affect the
; 				contents of the file.  Initially, the default
; 				tab settings set up by tabstops.ml have a tab
; 				stop every tab-size columns.
;FILES
;
;BUGS
;  There is no way to edit the default tab stop settings, except by
;  (setq-default tab-stops new-tabstop-template).
;
;HISTORY
; {1}	21-Dec-84  Mike Gallaher (mg) at unipress
; 	Created.
;
;----------------------------------------------------------------------

(if (! (is-bound &tab-buffer))
    (setq-default &tab-buffer "tabs"))
(if (! (is-bound &max-window-width))
    (setq-default &max-window-width (window-width)))

(declare-buffer-specific tab-stops)

(defun (&tab-stops-init
	(temp-use-buffer &tab-buffer)
	(erase-buffer)
	(if (is-bound tab-stops)
            (insert-string tab-stops)
	    (progn	count
		(setq count (+ (/ &max-window-width tab-size) 1))
		(while (-- count)
		    (insert-string "        T"))
	    ))
        (beginning-of-file)
	(set-mark)
	(end-of-file)
        (setq-default tab-stops (region-to-string))
        (novalue)
  ))

(defun (edit-tab-stops	       ts
    (setq ts tab-stops)
    (save-window-excursion
	(pop-to-buffer &tab-buffer)
	(erase-buffer)
	(insert-string ts)
	(setq overwrite-mode 1)
	(recursive-edit)
	(beginning-of-file)
	(set-mark)
	(end-of-file)
	(setq ts (region-to-string))
    )
    (setq tab-stops ts)
    (novalue)))

(defun (tab-to-tab-stop orig-pos  pos
    (if (= tab-stops "")
	 (insert-character '\t')
	 (progn
	     (setq orig-pos (current-column))
	     (save-excursion
	       (temp-use-buffer &tab-buffer)
	       (beginning-of-file)
	       (if (error-occurred
	             (provide-prefix-argument orig-pos (forward-character))
	           )
	         (end-of-line)
	       )
	       (if (error-occurred (search-forward "T"))
	          (end-of-line)
	 	 (backward-character))
	       (setq pos (current-column))
	     )
	     (to-col pos))
    )
    (novalue)
))

(local-bind-to-key "tab-to-tab-stop" "\^i")
(&tab-stops-init)
(novalue)
