;----------------------------------------------------------------------------
;NAME
;   tags -- twenex-like tags package
;
;SYNOPSIS
;   add-tag		-- add current line to database
;   goto-tag		-- goto location of given tag
;   make-tag-table	-- generate the database
;   recompute-all-tags	-- update the database
;   visit-function	-- visit the tag dot is in
;   visit-tag-table	-- use other than the normal tag table
;
;KEY BINDINGS
;   ^X^G is recommended by the author for goto-tag
;
;DESCRIPTION
;   The database used by the tag package (the 'tagfile') correlates function
;definitions  to  the  file  in  which  the definitions appear.  The primary
;function of the tag package is to allow the user to specify the name  of  a
;function,  and then have Emacs locate the definition of that function.  The
;commands implemented are:
;
;add-tag         Adds the current line (it should be the definition line for
;                some function) to the current tagfile.
;
;goto-tag        goto-tag  takes a single string argument which  is  usually
;                the  name of a function and visits the file containing that
;                function with the first line of the function at the top  of
;                the  window.  The string may actually be a substring of the
;                function name (actually, any substring of the first line of
;                the function definition).  If goto-tag is given  a  numeric
;                argument  then  rather than asking for a new string it will
;                use the old string and search for the  next  occurrence  of
;                that  string  in  the  tagfile.   This is used for stepping
;                through a set of tags that contain the same string.
;
;make-tag-table  Takes a list of file names  (with  wildcards  allowed)  and
;                builds a tagfile for all the functions in all of the files.
;                It determines the language of the contents of the file from
;                the  extension.    This  command  may take a while on large
;                directories, be prepared to wait.  A common use is to  type
;                "make-tag-table *.c".
;
;recompute-all-tags
;                Goes  through  your  current  tag  file  and  for each file
;                mentioned refinds all of the tags.  This is used to rebuild
;                an entire tag file if you've made very extensive changes to
;                the files mentioned and the tag package is no  longer  able
;                to  find functions.  The tagfile contains hints to help the
;                system locate the tagged function, as you make  changes  to
;                the   various   files   the   hints  become  out  of  date.
;                Periodically (not too  often!)  you  should  recompute  the
;                tagfile.
;
;visit-function  Takes  the  function name at or before dot, does a goto-tag
;                on that name, then puts you into a recursive-edit  to  look
;                at the function definition.  To get back to where you were,
;                just  type ^C.  This is used when you're editing something,
;                have dot positioned at some function invocation, then  want
;                to look at the function.
;
;visit-tag-table Normally  the name of the tagfile is ".tags" in the current
;                directory.    If  you  want  to  use  some  other  tagfile,
;                visit-tag-table lets you do that.
;
;NOTE
;   A tag file is a sequence of lines of the following items:
;
;   ^_filename
;   ^Atagline^Bposition
;
;   Each tagline/position pair refers to the preceeding file
;
;HISTORY
;{1}	14-Mar-85  David Medinets (dm) at unipress
;	added new variables &tag-buffer  and &tag-file,  also renamed the
;	fn variable to tag_fn.
;
;  [3]	4-Jul-84  Eric S. Raymond
;	Added documentation.
;  [2]	16-Jul-82 ?
;	This timestamp and the remark 'Edit 7' left by persons unknown
;  [1]	1-Nov-81  James Gosling
;	Created.
;----------------------------------------------------------------------------

(declare-global last-search-tag
				&tag-buffer
				&tag-file)

(setq &tag-buffer "*TAG*")
(setq &tag-file   ".tags")

(defun
	(to-tag-buffer
		(temp-use-buffer &tag-buffer)
		(if (& (= (buffer-size) 0) (= (current-file-name) ""))
			(progn
				(if (error-occurred (read-file &tag-file))
		       		(progn
			   			(write-named-file &tag-file)
			   			(message "New tag file")
                    )
                )
				(beginning-of-file)
            )
        )
    )
)

(defun
	(visit-tag-table tag_fn
		(setq tag_fn (arg 1 ": visit-tag-table "))
		(save-excursion
	    	(temp-use-buffer &tag-buffer)
	        (read-file tag_fn)
        )
    )
)

(defun
	(goto-tag tag_fn str pos restart
		(setq restart 0)
	    (if (! prefix-argument-provided)
	    	(progn
		   		(setq str (arg 1 "goto-tag: "))
				(if (!= str "")
					(progn
			    		(setq last-search-tag (concat "\^A*" str "\(*\)*"))
			    		(setq restart 1)
                     )
                )
        	)
        )
	    (save-excursion
	    	(to-tag-buffer)
	        (if restart
				(beginning-of-file)
            )
	        (re-search-forward last-search-tag)
	       	(beginning-of-line)
	       	(re-search-forward "\^A\\([^\^B]*\\)\^B\\(.*\\)")
	       	(region-around-match 1)
	       	(setq str (region-to-string))
	       	(region-around-match 2)
	       	(setq pos (- (region-to-string) 300))
	       	(save-excursion
		   	(re-search-reverse "\^_\\(.*\\)")
		   	(region-around-match 1)
		   	(setq tag_fn (region-to-string)))
        )
	   	(visit-file tag_fn)
	   	(goto-character pos)
	   	(if (error-occurred (search-forward str))
	    	(search-reverse "")
        )
	   	(beginning-of-line)
	   	(line-to-top-of-window)
	)
)

(defun
	(find-pos-str
		(beginning-of-line)
	   	(setq pos (+ (dot) 0))
	   	(set-mark)
	  	(end-of-line)
	   	(setq str (region-to-string))
    )
)

(defun
	(store-pos-str
		(insert-character '^A')
	    (insert-string str)
	    (insert-character '^B')
	    (insert-string pos)
	    (newline)
    )
)

(defun
	(add-tag
		(save-excursion pos str tag_fn
			(find-pos-str)
			(setq tag_fn (current-file-name))
			(to-tag-buffer)
			(beginning-of-file)
	       	(if (error-occurred (re-search-forward
                                  (concat "\^_" tag_fn "[^\^_]*")))
				(progn
		       		(beginning-of-file)
		       		(insert-character '^_')
		       		(insert-string tag_fn)
		       		(newline)
                )
            )
	       	(store-pos-str)
	       	(beginning-of-file)
        )
    )
)

(defun
	(add-tag* pos str
		(find-pos-str)
		(save-excursion
			(temp-use-buffer &tag-buffer)
			(store-pos-str)
        )
    )
)

(defun
	(add-all-tags pattern tag_fn
		(setq pattern (arg 1 ": add-all-tags (pattern) "))
		(setq tag_fn (current-file-name))
		(save-excursion
			(to-tag-buffer)
			(if (error-occurred (search-forward (concat "\^_" tag_fn "\n")))
				(progn
					(beginning-of-file)
					(insert-character '^_')
					(insert-string tag_fn)
					(newline)
                )
				(progn
			    	(set-mark)
			       	(while (= (following-char) '^A')
				    	(next-line)
	                )
			       	(erase-region)
	            )
            )
        )
	   	(save-excursion
	    	(error-occurred
		   		(beginning-of-file)
		   		(while 1
		       		(re-search-forward pattern)
		       		(add-tag*)
                )
            )
        )
		(novalue)
    )
)

(defun
	(add-typed-tags ext pattern
		(setq ext (substr (current-file-name) -2 2))
		(add-all-tags
			(if
				(= ext ".l") "^(def"
				(= ext ".c") "^[A-z].*(.*)"
				(= ext "ml") "^(defun[ \t\n]*("
				(= ext "ss") "@section\\|@chapter\\|@subsection"
				(= ext ".p") "function\\|procedure"
				(error (concat "Can't tag " (current-file-name)))
            )
        )
    )
)

(defun
	(tag-file tag_fn
		(setq tag_fn (arg 1 ": tag-file (filename) "))
		(message (concat "Tagging " tag_fn))
	   	(sit-for 20)
		(save-window-excursion
			(error-occurred
				(visit-file tag_fn)
				(add-typed-tags)
				(delete-buffer (current-buffer-name))
            )
        )
    )
)

(defun (recompute-all-tags
	   (save-window-excursion
	       (to-tag-buffer)
	       (beginning-of-file)
	       (error-occurred
		   (while 1
		       (re-search-forward "\^_\\(.*\\)")
		       (region-around-match 1)
		       (tag-file (region-to-string)))
	       )
	       (write-named-file &tag-file))))

(defun
	(make-tag-table fns
		(setq fns (arg 1 ": make-tag-table (from filenames) "))
	   	(message "Building tag table for " fns)
		(save-window-excursion
			(temp-use-buffer "*TEMP*")
	       	(erase-buffer)
	       	(set-mark)
	       	(filter-region (concat "ls " fns))
	       	(beginning-of-file)
	       	(while (! (eobp))
		    	(set-mark)
		      	(end-of-line)
		      	(delete-white-space)
		      	(tag-file (region-to-string))
		      	(next-line)
		      	(beginning-of-line)
            )
	       	(delete-buffer "*TEMP*")
	       	(temp-use-buffer &tag-buffer)
	     	(write-named-file &tag-file)
        )
   	   	(novalue)
    )
)

(defun (visit-function func
	   (save-window-excursion
	       (forward-character)
	       (backward-word)
	       (set-mark)
	       (forward-word)
	       (setq func (region-to-string))
	       (goto-tag func)
	       (message "Type ^C to go back")
	       (recursive-edit)
	   )
       ))

; tags.ml ends here ---------------------------------------------------------
