;--------------------------------------------------------------------------
;NAME
;   whist.ml -- function(s) for maintaining history logs
;
;SYNOPSIS
; get-arg
;	utility function to get a response from the keyboard.
; whist 
;	generate a log entry under the history section in a header.
; whist-abort
;	interrupt a whist message entry and leave.
; whist-find-history
;	 look for the history keyword in the file.
; whist-help
;	get help about this package.
; whist-get-file-type
; 	gets the type of file.
; whist-insert-cont-comment
;	places a whist-cont-comment and a tab before
;	every line in the file.
; whist-insert-edit-tag
; 	inserts an edit-tag for the next tag number.
; whist-next-tag-number
; 	returns the next number higher than the current tag.
; 
;KEY BINDINGS
;   None. Within the execution of whist itself, ^X^A is an abort.
;
;DESCRIPTION
;   This package implements the whist function to write history messages
; (just the same as whist on Unix). It knows how to generate properly time
; and user-stamped history entries for many different kinds of files. The
; history section of this source gives an example of its format.
;
;SEE ALSO
;   finger.ml -- finger is used to extract user's actual full-name
;
;HISTORY
; {9}	 5-Mar-85  David Medinets (dm) at unipress
;	Removed the references  to cont-comment-string  because they were
;	not  upward  compatible  with  previous  versions of  Emacs.  The
;	cont-comment  is  now  gotten  from whatever  is in  front of the
;	HISTORY keyword.
;
; {8}	 4-Oct-84  David Medinets (dm) at unipress
;	I  renamed whist-get-cont-comment  to whist-get-file-type because
;	you can now get the cont-comment-string directly  from the syntax
;	tables.  All of the history about comments  additions is obsolete
;	now.
; 
; {7}	 3-Oct-84  David Medinets (dm) at unipress
;	Rewrote whist-insert-edit-tag to include the  new functions begin
;	and end-comment-string.
;
; {6}	24-Sep-84  David Medinets (dm) at unipress
;	If an edit-tag is needed for a number other than  the next higher
;	history number, type ^Unn^X^T.  where nn  is the  edit tag number
;	to insert.
;
; {5}	18-Sep-84  David Medinets (dm) at unipress
;	Added code so that whist will not ask for an end comment in mlisp
;	files for the edit tags.
;
; {4}	17-Sep-84  David Medinets (dm) at unipress
;	Modified  the  edit  tag function  to ask  for the  begin and end-tag
;	strings if they  are null.   This  will have  to do  until the syntax
;	package works a little better.   Made  keep-going global  so that the
;	new  functions  can  exit  properly.     Also   made  some  variables
;	buffer-specific instead of global so that you can edit  with mlisp in
;	one  window and  C in  another.   Changed the  tag parens  to be user
;	definable.    Simply  set  whist-begin-tag  and  whist-end-tag to the
;	appropriate characters.  
;
; {3}	 7-Sep-84  David Medinets (dm) at unipress
;	I have created a function that will find the highest edit number,
;	add one to it and insert an edit-tag at the location of dot. It
;	is called whist-insert-edit-tag and is bound to the key ^X^T.
;	 There now 3 variables relating to comments:
;	    begin-comment = this is what starts the comment
;	        In the C language this is '/*'
;	    cont-comment  = this is what starts comment block lines
;	        In the C language this is ' *'
;	    end-comment   = This is what ends the comment
;	        In the C language this is '*/'
;	
;	Hopefully  these  will eventually  be able  to be  drawn from the
;	syntax tables for  each filetype.   I  have also  pulled the code
;	that decided which file-type a file  is into  a separate function
;	called whist-get-cont-comment.   This function  puts the filetype
;	into  a  global  var  called  whist-file-type  and  along the way
;	decides what cont-comment should contain.
; 
;  {2}	 7-Sep-84  David Medinets (dm) at unipress
;	Extracted the code to find the history message  and its attendent
;	error-processing so that any function can  use it.   The function
;	is called find-history and has no arguments.
;
;  {1}	31-Aug-84  David Medinets (dm) at unipress
;	The command 'whist' will now look inside the source file to find
;	a number in the form '[XX]' somewhere in the three lines after
;	the word 'history'.  If there is no number it will insert the
;	number [1] and if there is a number already there then it will
;	add one to it.
;
; 11-Jul-84  Eric S. Raymond (eric) at Rabbit Software
;     Improved finger-name to handle &-substitution and moved it to
;     its own file for common use, putting an autoload in this file.
;
;  4-Jul-84  Eric S. Raymond (eric) at Rabbit Software
;     Unbelievably, rxb had whist-help erasing the current buffer! This is
;     now fixed. The help message now contains a command synopsis. A
;          documentation header has been added to the source file. Paragraph
;      justification is now automatic on exit. Your site name is now from
;     (system-name) and your user-name will be correctly extracted from
;     a finger-format pw->gecos entry.    
;
;  3-Mar-83  Rafael Bracho (rxb) at Carnegie-Mellon University
;     Bug fixed so it doesn't add the tabs when writing history
;     messages in manual entries, troff actually processes them as
;     text and it comes out wrong. Also added file-type "?" so the
;     user may obtain immediate help.
;
; 26-Jan-83  Rafael Bracho (rxb) at Carnegie-Mellon University
;     Initial loading message taking out since emacs now pre-compiles
;     the code and thus it is usually very fast to load the package.
;
; 25-Jan-83  Rafael Bracho (rxb) at Carnegie-Mellon University
;     Small bug fixed that didn't prefix blank lines in the comment
;     messages.
;
; 13-Jan-83  James Gosling (jag) at Carnegie-Mellon University
;     Made a few minor changes before installing it in the public
;     library (removed some "concat" calls).
;
; 12-Jan-83  Rafael Bracho (rxb) at Carnegie-Mellon University
;     Internal definition of get-arg added in case the user doesn't
;     have my get-arg already loaded (quite possible since get-arg is
;     not in the standard library!). Sorry for all the people that
;     tried to use it before!
;
; 28-Dec-82  Rafael Bracho (rxb) at Carnegie-Mellon University
;     Help message for unknown file types added. Also "special" type
;     added to enable the user to specify the prefix string (a la -p
;     flag for whist(1)).
;
; 27-Dec-82  Rafael Bracho (rxb) at Carnegie-Mellon University
;     Changed so it is possible to abort on every level. Also
;     different actions included.
;
; 25-Dec-82  Rafael Bracho (rxb) at Carnegie-Mellon University
;     Created. This is actually the first real file the package is used
;     upon.
; 
;--------------------------------------------------------------------------

;  hopefully these variables will be removed by proper use of the syntax
;  tables.
(declare-global whist-begin-tag
                whist-end-tag
                keep-going
)
(declare-buffer-specific whist-file-type)

(setq whist-file-type     "")
(setq whist-begin-tag    "{")
(setq whist-end-tag      "}")

(if (error-occurred (path-of "finger.ml"))
   (error-message (concat "whist.ml needs a file called finger.ml"
                          " and it does not exist"))
   (autoload "finger-name" "finger.ml")
)

;
;      To get input from the user with a default (a la get* in C)
;     (in case you don't have a copy of my get-arg).
;

(defun 
  (get-arg return-argum
    (if (! (error-occurred 
              (setq return-argum (get-tty-string (arg 1) " [" (arg 2) "] "))
	   )
	)
        (if (= return-argum "") (setq return-argum (arg 2)))
    )
     return-argum
    )
)

;
;     To abort the run inside of recursive edits ...
;
(defun
   (whist-abort
     (setq keep-going 0)
     (setq buffer-is-modified 0)
     (exit-emacs)
    )
    
    
    ;
    ;     To provide help about whist
    ;
    (whist-help
     (setq needs-checkpointing 0)
     (pop-to-buffer "Help")
     (erase-buffer)
     (insert-string
         "The following commands are valid:\n"
         "   whist\t-- generate a history entry\n"
         "   whist-help\t-- get help about this package\n"
         "   whist-abort\t-- interrupt a message entry and leave\n\n"
         "The following file-types are recognized:\n\n"
         "ada\t\tassembler\tbc\t\tc\n"
         "csh\t\tfortran\t\tg\t\tlint\n"
         "lisp\t\tmakefile\tman\t\tmlisp\n"
         "nroff\t\tpascal\t\tscribe\t\tshell\n"
         "text\t\tyacc\n\n"
         "You may also specify ""special"" to enter "
         "a prefix string, or ""."" to abort."
     )
     (setq read-only 1)
    )


   ;
   ;     This is where everything happens ...
   ;
    
   (whist action old-ctrl-c old-ctrl-xa filename cont-comment
     (save-excursion
       (setq filename (current-file-name))
       (setq keep-going 1)
       (setq old-ctrl-c (local-binding-of "\^C"))
       (setq old-ctrl-xa (local-binding-of "\^X\^A"))
       (local-bind-to-key "exit-emacs" "\^C")
       (setq case-fold-search 0)
       (whist-find-history)
       (save-window-excursion
         (pop-to-buffer "whist")
         (setq needs-checkpointing 0)
         (erase-buffer)
         (setq mode-line-format (concat "History message for " filename "%M"))
         (setq right-margin 65)
         (setq left-margin 1)
         (use-abbrev-table "text-mode")
         (use-syntax-table "text-mode")
         (local-bind-to-key "exit-emacs" "\^C")
         (local-bind-to-key "whist-abort" "\^X\^A")
         (message "Enter text; ^X^A to abort, ^C when done.")
         (recursive-edit)
         (justify-paragraph)
         (local-bind-to-key old-ctrl-xa "\^X\^A")
         (if (! keep-going) (error-message "Bye..."))
            (while (= (setq action (substr (get-arg (concat "Action? (abort,"
                                                      " edit, type, update):")
                                            "update" 
                                           ) 1 1
				   )
		      )
                       "e"
		   )
               (local-bind-to-key "whist-abort" "\^X\^A")
               (recursive-edit)
               (local-bind-to-key old-ctrl-xa "\^X\^A")
               (if (! keep-going) (error-message "Bye..."))
	    )
            (if (= action "a") (error-message "Bye..."))
       )
       (whist-find-history)
       (backward-word)
       (set-mark)
       (beginning-of-line)
       (setq cont-comment (region-to-string))
       (whist-insert-cont-comment)
       (whist-find-history)
       (progn time-string str
	  (setq str (whist-next-tag-number)) ; this MUST be set here.
          (if (= whist-file-type "")
             (whist-get-file-type)
          )
          (if (= whist-file-type "man")
             (insert-string "\n.TP\n")
          )
          (setq time-string (current-time))
          (insert-string "\n" 
                         cont-comment
                         whist-begin-tag str whist-end-tag "\t"
                         (if (= whist-file-type "shell") """ " "")
                         (substr time-string 9 2)  "-"
                         (substr time-string 5 3)  "-"
                         (substr time-string -2 2) "  " (finger-name)
                         " (" (users-login-name) ") at " (system-name)
                         (if (= whist-file-type "shell") " """ "") "\n"
	  )
       )
      (yank-buffer "whist")
      (delete-buffer "whist")
      (local-bind-to-key old-ctrl-c "\^C")
     )
   )

; This function looks for a history string in the file.  If it is
; not found then a recursive edit is entered until the string is
; inserted into the file.

    (whist-find-history found-history
       (setq keep-going 1) 
       (setq found-history 0)
       (while (! found-history)
          (beginning-of-file)
          (if (error-occurred (search-forward "HISTORY"))
             (progn
                (local-bind-to-key "whist-abort" "\^X\^A")
                (message "Add comment containing ""HISTORY"""
                         " (^X^A to abort, ^C to resume)")
                (recursive-edit)
                (local-bind-to-key old-ctrl-xa "\^X\^A")
                (if (! keep-going) (error-message "Bye..."))
	     )
             (setq found-history 1)
	  )
       )
    (end-of-line)
    (novalue)
    )


;
; To obtain a valid whist filetype.
;
   (whist-get-file-type file-name
      (if (= whist-file-type "")
         (progn (setq file-name (current-file-name))
                (if (| (= (substr file-name -8 8) "makefile")
                       (= (substr file-name -8 8) "Makefile")
		    )
                   (setq whist-file-type "makefile")
                    (= (substr file-name -2 2) ".c")
                   (setq whist-file-type "c")
                    (= (substr file-name -2 2) ".h")
                   (setq whist-file-type "c")
                    (= (substr file-name -2 2) ".f")
                   (setq whist-file-type "fortran")
                    (= (substr file-name -2 2) ".s")
                   (setq whist-file-type "assembler")
                    (= (substr file-name -2 2) ".g")
                   (setq whist-file-type "g")
                    (= (substr file-name -2 2) ".y")
                   (setq whist-file-type "yacc")
                    (= (substr file-name -2 2) ".p")
                   (setq whist-file-type "pascal")
                    (= (substr file-name -2 2) ".l")
                   (setq whist-file-type "lisp")
                    (= (substr file-name -2 2) ".n")
                   (setq whist-file-type "nroff")
                    (= (substr file-name -2 2) ".t")
                   (setq whist-file-type "nroff")
                    (= (substr file-name -3 3) ".ml")
      
      ; continuing the long train of filetypes.
      
                   (setq whist-file-type "mlisp")
                    (= (substr file-name -3 3) ".bc")
                   (setq whist-file-type "bc")
                    (= (substr file-name -3 3) ".sh")
                   (setq whist-file-type "shell")
                    (= (substr file-name -4 4) ".csh")
                   (setq whist-file-type "csh")
                    (= (substr file-name -4 4) ".ada")
                   (setq whist-file-type "ada")
                    (= (substr file-name -4 4) ".mss")
                   (setq whist-file-type "scribe")
                    (= (substr file-name -4 4) ".tbl")
                   (setq whist-file-type "nroff")
                    (= (substr file-name -4 4) ".eqn")
                   (setq whist-file-type "nroff")
                    (= (substr file-name -4 4) ".teq")
                   (setq whist-file-type "nroff")
		)
      
      ;  the next loop sees if you have a man file.
      	    (if (= whist-file-type "")
               (progn loop-counter
                  (setq loop-counter 0)
                  (while (< loop-counter 10)
                     (if (= (substr file-name -2 2) (concat "." loop-counter))
                        (setq whist-file-type "man")
		     )
                     (setq loop-counter (+ loop-counter 1))
		  )
	       )
	    )
      ; If all of the above does not work then the default is text
            (if (= whist-file-type "") (setq whist-file-type "text"))
      
	    )
	 )
   )

; This function inserts the whist-cont-comment at the front
; of every line in the buffer.

   (whist-insert-cont-comment
    (save-window-excursion 
      (temp-use-buffer "whist")
      (beginning-of-file)
       (if (= whist-file-type "") (whist-get-file-type))
         (if (!= whist-file-type "man")
          (if (= whist-file-type "shell")
            (error-occurred (re-replace-string "^\\(.*\\)$" ":\t"" & """))
            (error-occurred (re-replace-string "^.*$" (concat cont-comment
                                                              "\t&")))
	  )
	 )
     (end-of-file)
     (if (!= (preceding-char) 10) (insert-string "\n"))
     (insert-string cont-comment)
    )
   )
; 
; This function will return the next edit tag number
;   all of this code was written by the first edit tag
;   and moved here by the third.
; 
   (whist-next-tag-number str old-ctrl-xa
      (save-excursion
         (setq old-ctrl-xa (local-binding-of "\^X\^A"))
         (whist-find-history)
         (set-mark)
         (provide-prefix-argument 3 (next-line))
         (narrow-region)                       
         (exchange-dot-and-mark)
         (set-mark)
         (if (error-occurred (search-forward whist-begin-tag))
            (setq str 0) 
            (progn        
               (set-mark)  
               (forward-word)              
               (setq str (region-to-string))
	    )
	 )
      (widen-region)
      )
   (+ str 1)
   )

; This function will return a string of the form             ; [4]
;  'begin-comment [number] end-comment'                      ; [4]
     
   (whist-insert-edit-tag cur-col old-ctrl-xa tag-num
      (setq keep-going 1)
      (setq cur-col (current-column))
      (setq old-ctrl-xa (local-binding-of "\^X\^A"))
      (if prefix-argument-provided                           ; {6}  
         (setq tag-num prefix-argument)			     ; {6}  
         (setq tag-num (whist-next-tag-number))		     ; {6}  
      )							     
      (insert-string 
         (begin-comment-string (current-syntax-table))        ; {7} 
         " " whist-begin-tag		                      ; {4} 
         tag-num                                              ; {6}  
         whist-end-tag " "                                    ; {4} 
         (end-comment-string (current-syntax-table))          ; {7} 
      )								
      (next-line)	
      (end-of-line) 
      (to-col cur-col)					      
      (novalue)
   )
)

(bind-to-key "whist-insert-edit-tag" "\^x\^t") 

; whist.ml ends here ------------------------------------------------------
